# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Common RPC service and API tools for Solum."""

import eventlet
import oslo_messaging as messaging
from oslo_serialization import jsonutils

import solum.common.context
from solum import objects
from solum import rpc


# NOTE(paulczar):
# Ubuntu 14.04 forces librabbitmq when kombu is used
# Unfortunately it forces a version that has a crash
# bug.  Calling eventlet.monkey_patch() tells kombu
# to use libamqp instead.
eventlet.monkey_patch()


class JsonPayloadSerializer(messaging.NoOpSerializer):
    @staticmethod
    def serialize_entity(context, entity):
        return jsonutils.to_primitive(entity, convert_instances=True)


class RequestContextSerializer(messaging.Serializer):

    def __init__(self, base=None):
        self._base = base or messaging.NoOpSerializer()

    def serialize_entity(self, context, entity):
        if not self._base:
            return entity
        return self._base.serialize_entity(context, entity)

    def deserialize_entity(self, context, entity):
        if not self._base:
            return entity
        return self._base.deserialize_entity(context, entity)

    def serialize_context(self, context):
        return context.to_dict()

    def deserialize_context(self, context):
        return solum.common.context.RequestContext.from_dict(context)


class Service(object):
    _server = None

    def __init__(self, topic, server, handlers):
        serializer = RequestContextSerializer(JsonPayloadSerializer())
        # TODO(asalkeld) add support for version='x.y'
        target = messaging.Target(topic=topic, server=server)
        self._server = rpc.get_server(target, handlers, serializer)

    def serve(self):
        objects.load()
        self._server.start()
        self._server.wait()


class API(object):
    def __init__(self, context=None, topic=None):
        serializer = RequestContextSerializer(JsonPayloadSerializer())
        self._context = context
        if topic is None:
            topic = ''
        target = messaging.Target(topic=topic)
        self._client = rpc.get_client(target,
                                      serializer=serializer)

    def _call(self, method, *args, **kwargs):
        return self._client.call(self._context, method, *args, **kwargs)

    def _cast(self, method, *args, **kwargs):
        self._client.cast(self._context, method, *args, **kwargs)

    def echo(self, message):
        self._cast('echo', message=message)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import datetime
import re

from oslo_log import log as logging
from oslo_serialization import jsonutils as json

import solum

LOG = logging.getLogger(__name__)


class UploaderBase(object):
    context = None
    original_file_path = None
    resource = None
    stage_id = None
    stage_name = None
    strategy = None

    def __init__(self, context, original_file_path, resource, stage_id,
                 stage_name):
        self.context = context
        self.original_file_path = original_file_path
        self.transformed_path = original_file_path + '.tf'
        self.resource = resource
        self.stage_id = stage_id
        self.stage_name = stage_name

    def upload_log(self):
        pass

    def transform_jsonlog(self):
        with open(self.original_file_path, 'r') as logfile:
            with open(self.transformed_path, 'w') as tflogfile:
                for line in logfile.readlines():
                    try:
                        # Log lines generated from Python logger
                        # has 'level name' followed by ' - '.
                        # Remove this before parsing the log line
                        patrn = ('^ERROR - |^DEBUG - |'
                                 '^WARN - |^CRITICAL - |^INFO - ')
                        reg = re.compile(patrn)
                        m = reg.match(line)
                        if m is not None:
                            line = line.replace(m.group(0), '')
                        json_line = json.loads(line, encoding='utf-8')
                        json_line['user'] = ''
                        if json_line.get('_user') == 'false':
                            # add a suffix to the tag to differentiate user
                            # logs from system logs.
                            json_line['user'] = '.system'
                        flatline = ("%(@timestamp)s "
                                    "solum.%(task)s.%(stage_id)s%(user)s "
                                    "%(message)s\n")
                        tflogfile.write(flatline % json_line)
                    except ValueError:
                        LOG.debug("Could not parse json line: %s", line)

    def write_userlog_row(self, location, strategy_info=None):
        ulog = solum.objects.registry.Userlog()
        now = datetime.datetime.utcnow()
        ulog.created_at = now
        ulog.updated_at = now
        ulog.resource_type = self.resource.type
        ulog.resource_uuid = self.stage_id
        ulog.project_id = self.resource.project_id
        ulog.location = location
        ulog.strategy = self.strategy
        if strategy_info is None:
            strategy_info = {}
        ulog.strategy_info = json.dumps(strategy_info)
        ulog.create(self.context)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
from unittest import mock

from oslo_utils import uuidutils


fakeAuthTokenHeaders = {'X-User-Id': u'773a902f022949619b5c2f32cd89d419',
                        'X-Roles': u'admin, ResellerAdmin, _member_',
                        'X-Project-Id': u'fake_project_id',
                        'X-Project-Name': 'fake_project_name',
                        'X-User-Name': 'test',
                        'X-Auth-Token': u'5588aebbcdc24e17a061595f80574376',
                        'X-Forwarded-For': u'10.10.10.10, 11.11.11.11',
                        'X-Service-Catalog': u'{test: 12345}',
                        'X-Auth-Url': 'fake_auth_url',
                        'X-Identity-Status': 'Confirmed',
                        'X-Domain-Name': 'domain',
                        'X-Project-Domain-Id': 'project_domain_id',
                        'X-User-Domain-Id': 'user_domain_id',
                        }


class FakePecanRequest(mock.Mock):

    def __init__(self, **kwargs):
        super(FakePecanRequest, self).__init__(**kwargs)
        self.host_url = 'http://test_url:8080/test'
        self.application_url = 'http://test_url:8080/test/'
        self.context = {}
        self.body = ''
        self.content_type = 'text/unicode'
        self.accept = None
        self.params = {}
        self.path = '/v1/services'
        self.headers = fakeAuthTokenHeaders
        self.environ = {}
        self.pecan = dict(content_type=None)
        self.query_string = ''

    def __setitem__(self, index, value):
        setattr(self, index, value)


class FakePecanResponse(mock.Mock):

    def __init__(self, **kwargs):
        super(FakePecanResponse, self).__init__(**kwargs)
        self.status = None
        self.status_code = 200


class FakeApp(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeApp, self).__init__(**kwargs)
        self.__tablename__ = 'app'
        self.__resource__ = 'apps'
        self.user_id = 'fake user id'
        self.project_id = 'fake_project_id'
        self.id = 'test_uuid'
        self.name = 'fakeapp'
        self.description = 'fake app used for testing'
        self.language_pack = 'test_lp'
        self.deleted = False
        self.source = {
            'repository': 'http://github.com/example/a.git',
            'revision': 'master',
            'repo_token': 'test-repo-token',
            'private': False,
            'private_ssh_key': 'test-private-ssh-key'
        }
        self.ports = [80]
        self.workflow_config = {
            'run_cmd': 'python ./main.py',
            'test_cmd': 'python ./tests.py',
        }
        self.stack_id = ''
        self.trigger_actions = ["unittest", "build", "deploy"]
        self.scale_config = dict()
        self.raw_content = "{\"repo_token\": \"test-repo-token\"}"

    def as_dict(self):
        return {
            'user_id': self.user_id,
            'project_id': self.project_id,
            'id': self.id,
            'name': self.name,
            'description': self.description,
            'language_pack': self.language_pack,
            'deleted': self.deleted,
            'source': self.source,
            'ports': self.ports,
            'workflow_config': self.workflow_config,
            'stack_id': self.stack_id,
            'raw_content': self.raw_content
        }


class FakeService(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeService, self).__init__(**kwargs)
        self.__tablename__ = 'service'
        self.__resource__ = 'services'
        self.user_id = 'fake user id'
        self.project_id = 'fake_project_id'
        self.uuid = 'test_uuid'
        self.id = 8
        self.name = 'james'
        self.service_type = 'not_this'
        self.description = 'amazing'
        self.tags = ['this', 'and that']
        self.read_only = True

    def as_dict(self):
        return dict(service_type=self.service_type,
                    user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    name=self.name,
                    tags=self.tags,
                    read_only=self.read_only,
                    description=self.description)


class FakeAuthProtocol(mock.Mock):

    def __init__(self, **kwargs):
        super(FakeAuthProtocol, self).__init__(**kwargs)
        self.app = FakeApp()
        self.config = ''


class FakeSensor(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeSensor, self).__init__(**kwargs)
        self.__tablename__ = 'sensor'
        self.__resource__ = 'sensors'
        self.user_id = 'test_user_id'
        self.project_id = 'fake_project_id'
        self.uuid = '54d54'
        self.id = 'test_id'
        self.name = 'test_name'
        self.value = '42'
        self.sensor_type = 'int'
        self.documentation = 'http://test_documentation.com/'
        self.description = 'test_description'
        self.target_resource = 'http://test_target_resource.com/'

    def as_dict(self):
        return dict(name=self.name,
                    user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    value=self.value,
                    sensor_type=self.sensor_type,
                    documentation=self.documentation,
                    description=self.description,
                    target_resource=self.target_resource)


class FakeExtension(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeExtension, self).__init__(**kwargs)
        self.__tablename__ = 'extension'
        self.__resource__ = 'extension'
        self.user_id = 'user_id'
        self.project_id = 'fake_project_id'
        self.uuid = '44du3dx'
        self.documentation = 'http://test_documentation.com'
        self.description = 'test_desc'
        self.id = 'test_id'
        self.name = 'test_name'
        self.version = '12.3'

    def as_dict(self):
        return dict(name=self.name,
                    user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    version=self.version,
                    documentation=self.documentation,
                    description=self.description)


class FakeExecution(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeExecution, self).__init__(**kwargs)
        self.__tablename__ = 'execution'
        self.__resource__ = 'execution'
        self.uuid = '44du3dx'
        self.id = 'test_id'
        self.pipeline_id = 'pipeline-1-2-3-4'

    def as_dict(self):
        return dict(pipeline_id=self.pipeline_id,
                    uuid=self.uuid,
                    id=self.id)


class FakeInfrastructureStack(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeInfrastructureStack, self).__init__(**kwargs)
        self.__tablename__ = 'build_farm'
        self.__resource__ = 'build_farms'
        self.user_id = 'user_id'
        self.project_id = 'fake_project_id'
        self.uuid = 'ceda0408-c93d-4772-abb2-18f65189d440'
        self.description = 'test_desc'
        self.id = 'test_id'
        self.name = 'test_name'
        self.image_id = '1234'
        self.heat_stack_id = '5678'

    def as_dict(self):
        return dict(name=self.name,
                    user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    image_id=self.image_id,
                    heat_stack_id=self.heat_stack_id,
                    description=self.description)


class FakeWorkflow(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeWorkflow, self).__init__(**kwargs)
        self.__tablename__ = 'workflow'
        self.__resource__ = 'workflows'
        self.id = ''
        self.deleted = False
        self.project_id = 'fake_project_id'
        self.user_id = 'fake_user_id'
        self.app_id = 'fake_app_id'
        self.wf_id = '1'
        self.source = 'fake_source'
        self.config = 'fake_config'
        self.actions = 'fake_actions'
        self.assembly = 1
        self.status = 'BUILT'

    def as_dict(self):
        return dict(user_id=self.user_id,
                    project_id=self.project_id,
                    app_id=self.app_id,
                    source=self.source,
                    config=self.config,
                    actions=self.actions,
                    status=self.status)


class FakeAssembly(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeAssembly, self).__init__(**kwargs)
        self.__tablename__ = 'assembly'
        self.__resource__ = 'assemblies'
        self.user_id = 'fake user id'
        self.project_id = 'fake_project_id'
        self.plan_uuid = 'fake plan uuid'
        self.uuid = 'test_uuid'
        self.id = 8
        self.name = 'faker'
        self.components = [FakeComponent()]
        self.status = 'BUILDING'
        self.application_uri = 'test_uri'
        now = datetime.datetime.utcnow()
        self.created_at = now
        self.updated_at = now
        self.workflow = ['unittest', 'build', 'deploy']
        self.image_id = 8

    def as_dict(self):
        return dict(user_id=self.user_id,
                    project_id=self.project_id,
                    application_uri=self.application_uri,
                    uuid=self.uuid,
                    id=self.id,
                    name=self.name,
                    created_at=self.created_at,
                    updated_at=self.updated_at,
                    status=self.status,
                    workflow=self.workflow)


class FakePipeline(mock.Mock):
    def __init__(self, **kwargs):
        super(FakePipeline, self).__init__(**kwargs)
        self.__tablename__ = 'pipeline'
        self.__resource__ = 'pipelines'
        self.user_id = 'fake user id'
        self.project_id = 'fake_project_id'
        self.plan_uuid = 'fake plan uuid'
        self.uuid = 'test_uuid'
        self.id = 8
        self.name = 'faker'
        self.trust_id = 'trust_worthy'

    def last_execution(self):
        return FakeExecution()

    def as_dict(self):
        return dict(user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    name=self.name)


class FakePlan(mock.Mock):
    def __init__(self, **kwargs):
        super(FakePlan, self).__init__(**kwargs)
        self.__tablename__ = 'plan'
        self.__resource__ = 'plans'
        self.raw_content = ({'name': 'faker',
                            'artifacts': [{'language_pack': 'test_lp'}]})
        self.user_id = 'fake user id'
        self.project_id = 'fake_project_id'
        self.uuid = 'test_uuid'
        self.id = 8
        self.name = 'faker'
        self.deploy_keys_uri = None
        self.trust_id = 'trust_worthy'

    def as_dict(self):
        return dict(raw_content=self.raw_content,
                    user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    name=self.name)

    def refined_content(self):
        if self.raw_content and self.uuid:
            self.raw_content['uuid'] = self.uuid
        return self.raw_content


class FakeImage(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeImage, self).__init__(**kwargs)
        self.__tablename__ = 'image'
        self.__resource__ = 'images'
        self.user_id = 'fake user id'
        self.project_id = 'fake_project_id'
        self.uuid = 'test_uuid'
        self.id = 8
        self.name = 'faker'
        self.source_uri = 'git://here'
        self.description = 'test_desc'
        self.base_image_id = '1-2-3-4'
        self.created_image_id = '7-2-3-4'
        self.image_format = 'docker'
        self.source_format = 'dib'
        self.artifact_type = None
        self.status = 'READY'
        self.external_ref = 'TempUrl'
        self.docker_image_name = 'tenant-name-ts-commit'

    def as_dict(self):
        return dict(user_id=self.user_id,
                    project_id=self.project_id,
                    base_image_id=self.base_image_id,
                    image_format=self.image_format,
                    created_image_id=self.created_image_id,
                    uuid=self.uuid,
                    id=self.id,
                    name=self.name,
                    source_uri=self.source_uri,
                    source_format=self.source_format,
                    description=self.description)


class FakeComponent(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeComponent, self).__init__(**kwargs)
        self.id = 1
        self.__tablename__ = 'component'
        self.__resource__ = 'components'
        self.name = 'james'
        self.service_type = 'not_this'
        self.description = 'amazing'
        self.assembly_id = '1'
        self.parent_component_id = '2'
        self.tags = ['this', 'and that']

    def as_dict(self):
        return dict(service_type=self.service_type,
                    id=self.id,
                    name=self.name,
                    tags=self.tags,
                    assembly_id=self.assembly_id,
                    parent_component_id=self.parent_component_id,
                    description=self.description)


class FakeOperation(mock.Mock):

    def __init__(self, **kwargs):
        super(FakeOperation, self).__init__(**kwargs)
        self.__tablename__ = 'operation'
        self.__resource__ = 'operations'
        self.user_id = 'fake_user_id'
        self.project_id = 'fake_project_id'
        self.uuid = 'fake_uuid'
        self.id = 8
        self.name = 'fake_name'
        self.description = 'fake_description'
        self.tags = ['this', 'and that']
        self.documentation = 'fake_documentation'
        self.target_resource = 'fake_target_resource'

    def as_dict(self):
        return dict(user_id=self.user_id,
                    project_id=self.project_id,
                    uuid=self.uuid,
                    id=self.id,
                    name=self.name,
                    tags=self.tags,
                    documentation=self.documentation,
                    target_resource=self.target_resource,
                    description=self.description)


class FakeAttributeDefinition(mock.Mock):

    def __init__(self, **kwargs):
        super(FakeAttributeDefinition, self).__init__(**kwargs)
        self.name = 'fake_attribute'
        self.type = 'attribute_definition'


class FakeUserlog(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeUserlog, self).__init__(**kwargs)
        self.resource_uuid = uuidutils.generate_uuid()
        self.resource_type = 'app'
        self.location = 'fake location'
        self.strategy = 'local'
        self.strategy_info = '{}'


class FakeParameter(mock.Mock):
    def __init__(self, **kwargs):
        super(FakeParameter, self).__init__(**kwargs)
        self.plan_id = 11
        self.user_defined_params = {'key': 'ab"cd'}
        self.sys_defined_params = {}



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Get tempUrl for DU stored in swift."""

import hashlib
import hmac
import sys
import time

if len(sys.argv) < 8:
    print('USAGE: python get-temp-url.py storage_host container'
          ' app_name account secret ttl protocol')
    sys.exit(1)

storage_host = sys.argv[1]
container = sys.argv[2]
app_name = sys.argv[3]
account = sys.argv[4]
secret = sys.argv[5]
ttl = sys.argv[6]
protocol = sys.argv[7]

method = 'GET'
expires = int(time.time() + int(ttl))

base = protocol + "://"
base += storage_host

path = '/v1'
path += "/" + account
path += "/" + container
path += "/" + app_name

hmac_body = '%s\n%s\n%s' % (method, expires, path)
sig = hmac.new(secret, hmac_body, hashlib.sha1).hexdigest()

print('%s%s?temp_url_sig=%s&temp_url_expires=%s' % (base, path, sig, expires))



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Config options for Solum Worker service."""


from oslo_config import cfg

SERVICE_OPTS = [
    cfg.StrOpt('topic',
               default='solum-worker',
               help='The queue to add build tasks to'),
    cfg.HostAddressOpt('host',
                       default='localhost',
                       help='The location of the build rpc queue'),
    cfg.StrOpt('handler',
               default='shell',
               help='The worker endpoint to employ'),
    cfg.StrOpt('task_log_dir',
               default='/var/log/solum/worker',
               help='The directory containing task log output.'),
    cfg.StrOpt('proj_dir',
               default='',
               help=('The directory containing the project\'s code, '
                     'especially the contrib directory.')),
    cfg.StrOpt('log_url_prefix',
               default='http://localhost/',
               help='The prefix of test log URL to be sent back'),
    cfg.StrOpt('log_upload_strategy',
               default='local',
               help=('Upload task log to central storage, using modules like '
                     'swift and local from solum/uploaders.')),
    cfg.StrOpt('log_upload_swift_container',
               default='solum-logs',
               help='The name of the Swift container to upload logs to.'),
    cfg.StrOpt('param_file_path',
               default='/tmp/solum',
               help='The path of param files to save to.'),
    cfg.StrOpt('image_storage',
               default='glance',
               help='Image storage backend. This includes images created '
               'for LanguagePacks and Deployment Units. Possible values are '
               'docker_registry, swift and glance.'),
    cfg.StrOpt('docker_reg_endpoint',
               default="127.0.0.1",
               help='Docker registry endpoint'),
    cfg.StrOpt('delete_local_cache',
               default="false",
               help='Delete cached docker images and git repos from '
               'the worker node after building languagepacks and deployment '
               'units. Valid options are true or false.'),
    cfg.StrOpt('region_name',
               default="RegionOne",
               help='Region name to use'),
    cfg.StrOpt('temp_url_secret',
               default="secret",
               help='Secret to use with temp url'),
    cfg.StrOpt('temp_url_protocol',
               default="http",
               help='Protocol to use with temp url. Options are '
                    'http/https'),
    cfg.StrOpt('temp_url_ttl',
               default="604800",
               help='TTL in seconds.'),
    cfg.StrOpt('lp_location_url',
               default="",
               help='url to the container where LPs are stored.'),
    cfg.StrOpt('operator_lp_download_strategy',
               default="swift-client",
               help='Options for downloading operator LPs.'
               'Possible values are "wget" or "swift-client"'),
    cfg.StrOpt('lp_operator_user',
               default="demo",
               help='LP operator username.'),
    cfg.StrOpt('lp_operator_password',
               default="solum",
               help='LP operator password.'),
    cfg.StrOpt('lp_operator_tenant_name',
               default="demo",
               help='LP operator tenant name.'),
    cfg.StrOpt('docker_daemon_url',
               default="unix://var/run/docker.sock",
               help='docker daemon url.'),
    cfg.IntOpt('container_mem_limit', default=0,
               help='max memory a container can consume. No limit by default'),
    cfg.IntOpt('docker_build_timeout', default=1800,
               help='max time a docker build can take. Default: 30 minutes'),
    cfg.StrOpt('rootwrap_config',
               default='/etc/solum/rootwrap.conf',
               help='Path to the rootwrap configuration file to use for '
                    'running commands as root.'),
]

opt_group = cfg.OptGroup(
    name='worker',
    title='Options for the solum-worker service')


def list_opts():
    yield opt_group, SERVICE_OPTS


cfg.CONF.register_group(opt_group)
cfg.CONF.register_opts(SERVICE_OPTS, opt_group)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlalchemy

from solum.objects import extension as abstract
from solum.objects.sqlalchemy import models as sql


class Extension(sql.Base, abstract.Extension):
    """Represent an extension in sqlalchemy."""

    __resource__ = 'extensions'
    __tablename__ = 'extension'
    __table_args__ = sql.table_args()

    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True,
                           autoincrement=True)
    uuid = sqlalchemy.Column(sqlalchemy.String(36))
    project_id = sqlalchemy.Column(sqlalchemy.String(36))
    user_id = sqlalchemy.Column(sqlalchemy.String(36))
    description = sqlalchemy.Column(sqlalchemy.String(255))
    name = sqlalchemy.Column(sqlalchemy.String(100))
    version = sqlalchemy.Column(sqlalchemy.String(16))
    documentation = sqlalchemy.Column(sqlalchemy.String(255))


class ExtensionList(abstract.ExtensionList):
    """Represent a list of extensions in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return ExtensionList(sql.model_query(context, Extension))



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.v1 import language_pack
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.language_pack_handler.LanguagePackHandler')
class TestLanguagePackController(base.BaseTestCase):
    def test_language_pack_get(self, handler_mock, resp_mock,
                               request_mock, mock_policy):
        mock_policy.return_value = True
        handler_get = handler_mock.return_value.get
        handler_get.return_value = fakes.FakeImage()
        language_pack_obj = language_pack.LanguagePackController(
            'test_id')
        result = language_pack_obj.get()
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(result)
        handler_get.assert_called_once_with('test_id')

    def test_lp_get_not_found(self, handler_mock, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        handler_get = handler_mock.return_value.get
        handler_get.side_effect = exception.ResourceNotFound(
            name='language_pack', image_id='test_id')
        language_pack_obj = language_pack.LanguagePackController(
            'test_id')
        language_pack_obj.get()
        self.assertEqual(404, resp_mock.status)
        handler_get.assert_called_once_with('test_id')

    def test_language_pack_delete_not_found(self, LanguagePackHandler,
                                            resp_mock, request_mock,
                                            mock_policy):
        mock_policy.return_value = True
        hand_delete = LanguagePackHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            name='language_pack', language_pack_id='test_id')
        obj = language_pack.LanguagePackController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_language_pack_delete_ok(self, LanguagePackHandler,
                                     resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = LanguagePackHandler.return_value.delete
        hand_delete.return_value = None
        obj = language_pack.LanguagePackController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(204, resp_mock.status)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.language_pack_handler.LanguagePackHandler')
class TestLanguagePacksController(base.BaseTestCase):
    def setUp(self):
        super(TestLanguagePacksController, self).setUp()
        objects.load()

    def test_language_packs_get_all(self, LanguagePackHandler,
                                    resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = LanguagePackHandler.return_value.get_all
        hand_get.return_value = []
        resp = language_pack.LanguagePacksController().get_all()
        hand_get.assert_called_with()
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp)

    def test_language_packs_post(self, LanguagePackHandler, resp_mock,
                                 request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo',
                       'source_uri': 'git@github.com/sample/a.git',
                       'lp_metadata': 'some metadata', 'lp_params': {}}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = LanguagePackHandler.return_value.create
        hand_create.return_value = fakes.FakeImage()
        language_pack.LanguagePacksController().post()
        del json_create['lp_metadata']
        del json_create['lp_params']
        hand_create.assert_called_with(json_create, 'some metadata', {})
        self.assertEqual(201, resp_mock.status)

    def test_language_packs_post_nodata(self, LanguagePackHandler, resp_mock,
                                        request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        hand_create = LanguagePackHandler.return_value.create
        hand_create.return_value = fakes.FakeImage()
        ret_val = language_pack.LanguagePacksController().post()
        faultstring = str(ret_val['faultstring'])
        self.assertEqual("Missing argument: \"data\"", faultstring)
        self.assertEqual(400, resp_mock.status)

    def test_language_packs_post_badname(self, LanguagePackHandler, resp_mock,
                                         request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'foo==',
                       'source_uri': 'git@github.com/sample/a.git',
                       'lp_metadata': 'some metadata'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = LanguagePackHandler.return_value.create
        hand_create.return_value = fakes.FakeImage()
        ret_val = language_pack.LanguagePacksController().post()
        faultstring = str(ret_val['faultstring'])
        error_msg = 'Names must only contain a-z,0-9,-,_'
        self.assertTrue(faultstring.endswith(error_msg))
        self.assertEqual(400, resp_mock.status)

    def test_language_packs_post_capsname(self, LanguagePackHandler, resp_mock,
                                          request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'Foo',
                       'source_uri': 'git@github.com/sample/a.git',
                       'lp_metadata': 'some metadata'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = LanguagePackHandler.return_value.create
        hand_create.return_value = fakes.FakeImage()
        ret_val = language_pack.LanguagePacksController().post()
        faultstring = str(ret_val['faultstring'])
        error_msg = 'Names must only contain a-z,0-9,-,_'
        self.assertTrue(faultstring.endswith(error_msg))
        self.assertEqual(400, resp_mock.status)

    def test_language_packs_post_longname(self, LanguagePackHandler, resp_mock,
                                          request_mock, mock_policy):
        mock_policy.return_value = True
        json_create = {'name': 'a' * 101,
                       'source_uri': 'git@github.com/sample/a.git',
                       'lp_metadata': 'some metadata'}
        request_mock.body = json.dumps(json_create)
        request_mock.content_type = 'application/json'
        hand_create = LanguagePackHandler.return_value.create
        hand_create.return_value = fakes.FakeImage()
        ret_val = language_pack.LanguagePacksController().post()
        faultstring = str(ret_val['faultstring'])
        error_msg = 'Names must not be longer than 100 characters'
        self.assertTrue(faultstring.endswith(error_msg))
        self.assertEqual(400, resp_mock.status)



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""empty message

Revision ID: e5f2e8af3b8
Revises: 112cfe038dd3
Create Date: 2015-09-23 18:49:34.292533

"""
from alembic import op

# revision identifiers, used by Alembic.
revision = 'e5f2e8af3b8'
down_revision = '112cfe038dd3'


def upgrade():
    op.create_unique_constraint("app_id_wf_id", "workflow",
                                ['app_id', 'wf_id'])


def downgrade():
    op.drop_unique_constraint("app_id_wf_id", "workflow")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_concurrency import processutils as putils
from oslo_utils import strutils

from solum import privileged


# Entrypoint used for rootwrap.py transition code.  Don't use this for
# other purposes, since it will be removed when we think the
# transition is finished.
def execute(*cmd, **kwargs):
    run_as_root = kwargs.pop('run_as_root', False)
    kwargs.pop('root_helper', None)

    try:
        if run_as_root:
            return execute_root(*cmd, **kwargs)
        else:
            return putils.execute(*cmd, **kwargs)
    except OSError as e:
        sanitized_cmd = strutils.mask_password(' '.join(cmd))
        raise putils.ProcessExecutionError(
            cmd=sanitized_cmd, description=str(e))


@privileged.default.entrypoint
def execute_root(*cmd, **kwargs):
    return putils.execute(*cmd, shell=False, run_as_root=False, **kwargs)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


def get_network_parameters(osc):
    # TODO(julienvey) In the long term, we should have optional parameters
    # if the user wants to override this default behaviour
    params = {}
    tenant_networks = osc.neutron().list_networks()
    for tenant_network in tenant_networks['networks']:
        if tenant_network['router:external']:
            params['public_net'] = tenant_network['id']
        else:
            params['private_net'] = tenant_network['id']
            # Note (devkulkarni): Neutron subnet may contain
            # ipv6 and ipv4 subnets. We want to pick the ipv4 subnet
            params['private_subnet'] = get_ipv4_subnet_id(osc, tenant_network)
    return params


def get_ipv4_subnet_id(osc, tenant_network):
    for tenant_sub_id in tenant_network['subnets']:
        subnet_data = osc.neutron().show_subnet(tenant_sub_id)
        if subnet_data['subnet']['ip_version'] == 4:
            return tenant_sub_id



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import core
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import format
from solum.api.controllers.camp.v1_1.datamodel import formats
from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.handlers.camp import format_handler
from solum.common import exception


DESCRIPTION_STRING = "Solum CAMP API supported formats collection resource."


class FormatsController(rest.RestController):
    """Manages operations on CAMP's formats resource."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(format.Format, wtypes.text)
    def get_one(self, format_name):
        """Return the appropriate format resource."""
        handler = (format_handler.
                   FormatHandler(pecan.request.security_context))
        raw_format = handler.get(format_name)
        if not raw_format:
            core.abort(404,
                       '%s is not a format resource' %
                       format_name)
        host_url = pecan.request.application_url.rstrip('/')
        return raw_format.fix_uris(host_url)

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(formats.Formats)
    def get(self):
        """Return the formats resource."""
        host_url = pecan.request.application_url.rstrip('/')
        links = [common_types.Link(href=uris.JSON_FORMAT_URI_STR % host_url,
                                   target_name=uris.JSON_FORMAT_NAME_STR)]
        return formats.Formats(uri=uris.FORMATS_URI_STR % host_url,
                               name='Solum_CAMP_formats',
                               type='formats',
                               description=DESCRIPTION_STRING,
                               format_links=links)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import pecan
from pecan import rest
from urllib import parse
from wsme.rest import json as wsme_json
from wsme import types as wsme_types
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import assemblies as model
from solum.api.controllers.camp.v1_1 import uris
from solum.api.controllers import common_types
from solum.api.handlers.camp import assembly_handler
from solum.api.handlers.camp import plan_handler
from solum.common import exception


class AssembliesController(rest.RestController):
    """CAMP v1.1 assemblies controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(None, wsme_types.text, status_code=204)
    def delete(self, uuid):
        """Delete this assembly."""
        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        handler.delete(uuid)

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.Assemblies)
    def get(self):
        host_url = pecan.request.application_url.rstrip('/')
        auri = uris.ASSEMS_URI_STR % host_url
        pdef_uri = uris.DEPLOY_PARAMS_URI % host_url
        desc = "Solum CAMP API assemblies collection resource"

        handler = (assembly_handler.
                   AssemblyHandler(pecan.request.security_context))
        asem_objs = handler.get_all()
        a_links = []
        for m in asem_objs:
            a_links.append(common_types.Link(href=uris.ASSEM_URI_STR %
                                             (host_url, m.uuid),
                                             target_name=m.name))

        # if there aren't any assemblies, avoid returning a resource with an
        # empty assembly_links array
        if len(a_links) > 0:
            res = model.Assemblies(uri=auri,
                                   name='Solum_CAMP_assemblies',
                                   type='assemblies',
                                   description=desc,
                                   parameter_definitions_uri=pdef_uri,
                                   assembly_links=a_links)
        else:
            res = model.Assemblies(uri=auri,
                                   name='Solum_CAMP_assemblies',
                                   type='assemblies',
                                   description=desc,
                                   parameter_definitions_uri=pdef_uri)

        return res

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.Assembly, wsme_types.text)
    def get_one(self, uuid):
        """Return the appropriate CAMP-style assembly resource."""
        host_url = pecan.request.application_url.rstrip('/')
        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        return model.Assembly.from_db_model(handler.get(uuid),
                                            host_url)

    @exception.wrap_pecan_controller_exception
    @pecan.expose('json', content_type='application/json')
    def post(self):
        """Create a new application.

        There are a number of ways to use this method to create a new
        application. See Section 6.11 of the CAMP v1.1 specification
        for an explanation of each. Use the Content-Type of request to
        determine what the client is trying to do.
        """
        if pecan.request.content_type is None:
            raise exception.UnsupportedMediaType(
                name=pecan.request.content_type,
                method='POST')

        req_content_type = pecan.request.content_type
        host_url = pecan.request.application_url.rstrip('/')

        # deploying by reference uses a JSON payload
        if req_content_type == 'application/json':
            payload = pecan.request.body
            if not payload or len(payload) < 1:
                raise exception.BadRequest(reason='empty request body')

            try:
                json_ref_doc = json.loads(payload)
            except ValueError as excp:
                raise exception.BadRequest(reason='JSON object is invalid. '
                                           + str(excp))

            if 'plan_uri' in json_ref_doc:
                plan_uri_str = json_ref_doc['plan_uri']

                # figure out if the plan uri is relative or absolute
                plan_uri = parse.urlparse(plan_uri_str)
                uri_path = plan_uri.path
                if not plan_uri.netloc:
                    # should be something like "../plans/<uuid>" or
                    # "/camp/v1_1/plans/<uuid> (include Solum plans)
                    if (not uri_path.startswith('../plans/') and
                            not uri_path.startswith('../../../v1/plans/') and
                            not uri_path.startswith('/camp/v1_1/plans/') and
                            not uri_path.startswith('/v1/plans/')):
                        msg = 'plan_uri does not reference a plan resource'
                        raise exception.BadRequest(reason=msg)

                    plan_uuid = plan_uri.path.split('/')[-1]

                else:
                    # We have an absolute URI. Try to figure out if it refers
                    # to a plan on this Solum instance. Note the following code
                    # does not support URI aliases. A request that contains
                    # a 'plan_uri' with a network location that is different
                    # than network location used to make this request but
                    # which, nevertheless, still refers to this Solum instance
                    # will experience a false negative. This code will treat
                    # that plan as if it existed on another CAMP-compliant
                    # server.
                    if plan_uri_str.startswith(host_url):
                        if (not uri_path.startswith('/camp/v1_1/plans/') and
                                not uri_path.startswith('/v1/plans/')):
                            msg = 'plan_uri does not reference a plan resource'
                            raise exception.BadRequest(reason=msg)

                        plan_uuid = plan_uri.path.split('/')[-1]

                    else:
                        # The plan exists on another server.
                        # TODO(gpilz): support references to plans on other
                        # servers
                        raise exception.NotImplemented()

                # resolve the local plan by its uuid. this will raise a
                # ResourceNotFound exception if there is no plan with
                # this uuid
                phandler = plan_handler.PlanHandler(
                    pecan.request.security_context)
                plan_obj = phandler.get(plan_uuid)

            elif 'pdp_uri' in json_ref_doc:
                # TODO(gpilz): support references to PDPs
                raise exception.NotImplemented()
            else:
                # must have either 'plan_uri' or 'pdp_uri'
                msg = 'JSON object must contain either plan_uri or pdp_uri'
                raise exception.BadRequest(reason=msg)
        else:
            # TODO(gpilz): support deploying an application by value
            raise exception.NotImplemented()

        # at this point we expect to have a reference to a plan database object
        # for the plan that will be used to create the application
        ahandler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        assem_db_obj = ahandler.create_from_plan(plan_obj)
        assem_model = model.Assembly.from_db_model(assem_db_obj, host_url)

        pecan.response.status = 201
        pecan.response.location = assem_model.uri
        return wsme_json.tojson(model.Assembly, assem_model)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.api.handlers import plan_handler as solum_plan_handler
from solum import objects


class PlanHandler(solum_plan_handler.PlanHandler):
    def delete(self, id):
        """Override to simply delete the appropriate row in the DB.

        This will raise an exception if any assemblies refer to this plan.
        """
        db_obj = objects.registry.Plan.get_by_uuid(self.context, id)
        self._delete_params(db_obj.id)
        db_obj.destroy(self.context)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from solum.api.controllers.camp.v1_1 import assemblies
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.camp.assembly_handler.AssemblyHandler')
@mock.patch('solum.api.handlers.camp.plan_handler.PlanHandler')
class TestAssemblies(base.BaseTestCase):
    def setUp(self):
        super(TestAssemblies, self).setUp()
        objects.load()

    def test_assemblies_get(self, PlanHandler, AssemblyHandler, resp_mock,
                            request_mock):
        hand_get_all = AssemblyHandler.return_value.get_all
        fake_assembly = fakes.FakeAssembly()
        hand_get_all.return_value = [fake_assembly]

        resp = assemblies.AssembliesController().get()
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertIsNotNone(resp['result'].assembly_links)
        assembly_links = resp['result'].assembly_links
        self.assertEqual(1, len(assembly_links))
        self.assertEqual(fake_assembly.name, assembly_links[0].target_name)

    def test_assemblies_post_no_content_type(self, PlanHandler,
                                             AssemblyHandler, resp_mock,
                                             request_mock):
        # creating an assembly requires a Content-Type so the CAMP impl
        # can figure out which mechanism the user wants
        request_mock.content_type = None

        assemblies.AssembliesController().post()
        self.assertEqual(415, resp_mock.status)

    def test_assemblies_post_ref_none(self, PlanHandler, AssemblyHandler,
                                      resp_mock, request_mock):
        # a Content-Type of 'application/json' indicates that the user is
        # providing a JSON object that references either a plan or a PDP
        request_mock.content_type = 'application/json'
        request_mock.body = None

        assemblies.AssembliesController().post()
        self.assertEqual(400, resp_mock.status)

    def test_assemblies_post_ref_empty_json(self, PlanHandler, AssemblyHandler,
                                            resp_mock, request_mock):
        # a Content-Type of 'application/json' indicates that the user is
        # providing a JSON object that references either a plan or a PDP
        request_mock.content_type = 'application/json'
        request_mock.body = '{}'

        assemblies.AssembliesController().post()
        self.assertEqual(400, resp_mock.status)

    def test_assemblies_post_ref_bad_rel_uri(self, PlanHandler,
                                             AssemblyHandler, resp_mock,
                                             request_mock):
        # a Content-Type of 'application/json' indicates that the user is
        # providing a JSON object that references either a plan or a PDP
        ref_object = {'plan_uri':
                      '../fooble/24e3974c-195d-4a6a-96b0-7924ed3c742a'}
        request_mock.content_type = 'application/json'
        request_mock.body = json.dumps(ref_object)

        assemblies.AssembliesController().post()
        self.assertEqual(400, resp_mock.status)

    def test_assemblies_post_ref_rel_uris(self, PlanHandler, AssemblyHandler,
                                          resp_mock, request_mock):
        hand_get = PlanHandler.return_value.get
        hand_get.return_value = fakes.FakePlan()

        hand_create_from_plan = AssemblyHandler.return_value.create_from_plan
        fake_assembly = fakes.FakeAssembly()
        hand_create_from_plan.return_value = fake_assembly

        cntrl = assemblies.AssembliesController()

        # test reference to CAMP plan relative to assemblies
        ref_object = {'plan_uri':
                      '../plans/24e3974c-195d-4a6a-96b0-7924ed3c742a'}
        request_mock.content_type = 'application/json'
        request_mock.body = json.dumps(ref_object)

        resp = cntrl.post()
        self.assertIsNotNone(resp)
        self.assertEqual(201, resp_mock.status)
        self.assertIsNotNone(resp_mock.location)
        self.assertEqual(fake_assembly.name, resp['name'])

        # test reference to Solum plan relative to assemblies
        ref_object = {'plan_uri':
                      '../../../v1/plans/24e3974c-195d-4a6a-96b0-7924ed3c742a'}
        request_mock.content_type = 'application/json'
        request_mock.body = json.dumps(ref_object)

        resp = cntrl.post()
        self.assertIsNotNone(resp)
        self.assertEqual(201, resp_mock.status)
        self.assertIsNotNone(resp_mock.location)
        self.assertEqual(fake_assembly.name, resp['name'])

        # test reference to CAMP plan relative to root
        ref_object = {'plan_uri':
                      '/camp/v1_1/plans/24e3974c-195d-4a6a-96b0-7924ed3c742a'}
        request_mock.content_type = 'application/json'
        request_mock.body = json.dumps(ref_object)

        resp = cntrl.post()
        self.assertIsNotNone(resp)
        self.assertEqual(201, resp_mock.status)
        self.assertIsNotNone(resp_mock.location)
        self.assertEqual(fake_assembly.name, resp['name'])

        # test reference to Solum plan relative to root
        ref_object = {'plan_uri':
                      '/v1/plans/24e3974c-195d-4a6a-96b0-7924ed3c742a'}
        request_mock.content_type = 'application/json'
        request_mock.body = json.dumps(ref_object)

        resp = cntrl.post()
        self.assertIsNotNone(resp)
        self.assertEqual(201, resp_mock.status)
        self.assertIsNotNone(resp_mock.location)
        self.assertEqual(fake_assembly.name, resp['name'])



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import core
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import (parameter_definitions
                                                       as model)
from solum.api.handlers.camp import parameter_definitions_handler
from solum.common import exception


class ParamsDefController(rest.RestController):
    """Manages operations on a singular parameter_definitions resource."""

    def __init__(self, param_defs_name):
        super(ParamsDefController, self).__init__()
        self._id = param_defs_name

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.ParameterDefinitions)
    def get(self):
        """Return the appropriate parameter_definitions resource."""
        handler = (parameter_definitions_handler.
                   ParameterDefinitionsHandler(pecan.request.security_context))
        raw_defs = handler.get(self._id)
        if not raw_defs:
            core.abort(404,
                       '%s is not a parameter_definitions collection' %
                       self._id)
        host_url = pecan.request.application_url.rstrip('/')
        return raw_defs.fix_uris(host_url)


class ParameterDefinitionsController(rest.RestController):
    """Manages operations on CAMP's parameter_definitions resources."""

#    @pecan.expose()
#    def _lookup(self, param_defs_name, *remainder):
#        if remainder and not remainder[-1]:
#            remainder = remainder[:-1]
#        return ParamsDefController(param_defs_name), remainder

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.ParameterDefinitions, wtypes.text)
    def get_one(self, param_defs_name):
        """Return the appropriate format resource."""
        handler = (parameter_definitions_handler.
                   ParameterDefinitionsHandler(pecan.request.security_context))
        raw_defs = handler.get(param_defs_name)
        if not raw_defs:
            core.abort(404,
                       '%s is not a parameter_definitions collection' %
                       param_defs_name)
        host_url = pecan.request.application_url.rstrip('/')
        return raw_defs.fix_uris(host_url)



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
import pecan
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp import root as camp_root
from solum.api.controllers import common_types
from solum.api.controllers.v1 import root as v1_root


STATUS_KIND = wtypes.Enum(str, 'SUPPORTED', 'CURRENT', 'DEPRECATED')

camp_group = cfg.OptGroup(name='camp',
                          title='Options that apply to the CAMP API')

CAMP_API_OPTS = [
    cfg.BoolOpt('camp_enabled',
                default=True,
                help=("Enable/disable support for the OASIS CAMP "
                      "API. Default value is True."))
]


def list_opts():
    yield camp_group, CAMP_API_OPTS


CONF = cfg.CONF
CONF.register_group(camp_group)
CONF.register_opts(CAMP_API_OPTS, group=camp_group)


class Version(wtypes.Base):
    """Version representation."""

    id = wtypes.text
    "The version identifier."

    status = STATUS_KIND
    "The status of the API (SUPPORTED, CURRENT or DEPRECATED)."

    link = common_types.Link
    "The link to the versioned API."

    @classmethod
    def sample(cls):
        return cls(id='v1.0',
                   status='CURRENT',
                   link=common_types.Link(target_name='v1',
                                          href='http://example.com:9777/v1'))


class RootController(object):

    v1 = v1_root.Controller()

    if CONF.camp.camp_enabled:
        camp = camp_root.Controller()

    @wsme_pecan.wsexpose([Version])
    def index(self):
        base_url = pecan.request.application_url.rstrip('/')
        host_url = '%s/%s' % (base_url, 'v1')
        v1 = Version(id='v1.0',
                     status='CURRENT',
                     link=common_types.Link(target_name='v1',
                                            href=host_url))
        return [v1]



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Move triggers from Assembly to Plan

Revision ID: 5583c6e70156
Revises: 1f57763d7871
Create Date: 2015-03-09 16:31:28.130484

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '5583c6e70156'
down_revision = '1f57763d7871'


def upgrade():
    op.add_column('plan',
                  sa.Column('trigger_id', sa.String(length=36)))
    op.add_column('plan',
                  sa.Column('trust_id', sa.String(length=255)))
    op.add_column('plan',
                  sa.Column('username', sa.String(length=256)))
    op.drop_column('assembly', 'trigger_id')
    op.drop_column('assembly', 'trust_id')


def downgrade():
    op.add_column('assembly',
                  sa.Column('trigger_id', sa.String(length=36)))
    op.add_column('assembly',
                  sa.Column('trust_id', sa.String(length=255)))
    op.drop_column('plan', 'trigger_id')
    op.drop_column('plan', 'trust_id')
    op.drop_column('plan', 'username')



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from wsme import types as wtypes


Uri = wtypes.text


class Link(wtypes.Base):
    """A link representation."""

    href = Uri
    "The link URI."

    target_name = wtypes.text
    "Textual name of the target link."

    @classmethod
    def sample(cls):
        return cls(href=('http://example.com:9777/v1'),
                   target_name='v1')



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.datamodel import platform_endpoint as model
from solum.api.controllers.camp.v1_1 import uris
from solum.common import exception


URI_STRING = '%s/camp/camp_v1_1_endpoint'
NAME_STRING = 'Solum_CAMP_v1_1_endpoint'
DESCRIPTION_STRING = "Solum CAMP v1.1 API platform_endpoint resource."


class CAMPv11EndpointController(rest.RestController):
    """camp_v1_1_endpoint resource controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.PlatformEndpoint)
    def get(self):
        host_url = pecan.request.application_url.rstrip('/')
        return model.PlatformEndpoint(uri=URI_STRING % host_url,
                                      name=NAME_STRING,
                                      type='platform_endpoint',
                                      description=DESCRIPTION_STRING,
                                      platform_uri=uris.PLATFORM_URI_STR %
                                      host_url,
                                      specification_version='CAMP 1.1',
                                      implementation_version='Solum CAMP 1.1',
                                      auth_scheme='KEYSTONE-2.0')



# Copyright 2014 - Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlalchemy as sa
from sqlalchemy.orm import exc

from solum.objects import parameter as abstract
from solum.objects.sqlalchemy import models as sql


class Parameter(sql.Base, abstract.Parameter):
    """Represent a parameter in sqlalchemy."""

    __resource__ = 'parameters'
    __tablename__ = 'parameter'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    plan_id = sa.Column(sa.Integer, sa.ForeignKey('plan.id'),
                        nullable=False)
    user_defined_params = sa.Column(sql.YAMLEncodedDict(65535))
    sys_defined_params = sa.Column(sql.YAMLEncodedDict(65535))

    @classmethod
    def get_by_plan_id(cls, context, p_id):
        try:
            session = sql.SolumBase.get_session()
            return session.query(cls).filter_by(plan_id=p_id).one()
        except exc.NoResultFound:
            return None


class ParameterList(abstract.ParameterList):
    """Represent a list of parameters in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return ParameterList(sql.model_query(context, Parameter))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_utils import uuidutils

from solum.api.handlers import assembly_handler as solum_assem_handler
from solum import objects


class AssemblyHandler(solum_assem_handler.AssemblyHandler):
    def create_from_plan(self, plan_obj):
        """Create an application using a plan resource as a template."""
        db_obj = objects.registry.Assembly()
        db_obj.uuid = uuidutils.generate_uuid()
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.tenant
        db_obj.trigger_id = uuidutils.generate_uuid()
        db_obj.username = self.context.user_name

        # use the plan name as the name of this application
        db_obj.name = plan_obj.name + "_application"
        db_obj.plan_id = plan_obj.id
        db_obj.plan_uuid = plan_obj.uuid

        db_obj.status = solum_assem_handler.ASSEMBLY_STATES.QUEUED
        db_obj.create(self.context)
        artifacts = plan_obj.raw_content.get('artifacts', [])

        # build each artifact in the plan
        for arti in artifacts:
            self._build_artifact(assem=db_obj, artifact=arti)

        return db_obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan import rest

from solum.api.controllers.camp.v1_1 import assemblies
from solum.api.controllers.camp.v1_1 import attribute_definitions
from solum.api.controllers.camp.v1_1 import extensions
from solum.api.controllers.camp.v1_1 import formats
from solum.api.controllers.camp.v1_1 import parameter_definitions
from solum.api.controllers.camp.v1_1 import plans
from solum.api.controllers.camp.v1_1 import platform
from solum.api.controllers.camp.v1_1 import services
from solum.api.controllers.camp.v1_1 import type_definitions


class Controller(rest.RestController):
    """CAMP API version 1.1 controller root."""

    assemblies = assemblies.AssembliesController()
    attribute_definitions = (attribute_definitions.
                             AttributeDefinitionsController())
    extensions = extensions.ExtensionsController()
    formats = formats.FormatsController()
    parameter_definitions = (parameter_definitions.
                             ParameterDefinitionsController())
    plans = plans.PlansController()
    platform = platform.PlatformController()
    services = services.ServicesController()
    type_definitions = type_definitions.TypeDefinitionsController()



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""empty message

Revision ID: 112cfe038dd3
Revises: 390c6c4aa97b
Create Date: 2015-08-20 19:11:01.544203

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '112cfe038dd3'
down_revision = '390c6c4aa97b'


def upgrade():
    op.add_column('app',
                  sa.Column('app_url', sa.String(length=1024)))
    op.add_column('app',
                  sa.Column('status', sa.String(length=36)))


def downgrade():
    op.drop_column('app', 'app_url')
    op.drop_column('app', 'status')



# Copyright (c) 2014 Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.handlers import plan_handler
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry')
class TestPlanHandler(base.BaseTestCase):
    def setUp(self):
        super(TestPlanHandler, self).setUp()
        self.ctx = utils.dummy_context()

    def test_plan_get(self, mock_registry):
        mock_registry.Plan.get_by_uuid.return_value = {}
        handler = plan_handler.PlanHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        get_by_uuid = mock_registry.Plan.get_by_uuid
        get_by_uuid.assert_called_once_with(self.ctx, 'test_id')

    def test_plan_get_all(self, mock_registry):
        mock_registry.PlanList.get_all.return_value = {}
        handler = plan_handler.PlanHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.PlanList.get_all.assert_called_once_with(self.ctx)

    def test_plan_update(self, mock_registry):
        data = {'user_id': 'new_user_id', 'name': 'new_name',
                'project_id': 'new_proj_id', 'uuid': 'new_uuid'}
        db_obj = fakes.FakePlan()
        mock_registry.Plan.get_by_uuid.return_value = db_obj
        db_obj.raw_content.update(data)
        to_update_data = {'name': data['name'],
                          'raw_content': db_obj.raw_content}
        handler = plan_handler.PlanHandler(self.ctx)
        handler.update('test_id', data)
        mock_registry.Plan.update_and_save.assert_called_once_with(
            self.ctx, 'test_id', to_update_data)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_plan_create(self, mock_kc, mock_registry):
        data = {'name': 'new_name',
                'uuid': 'input_uuid'}
        db_obj = fakes.FakePlan()
        mock_registry.Plan.return_value = db_obj
        handler = plan_handler.PlanHandler(self.ctx)
        res = handler.create(data)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_plan_create_with_param(self, mock_kc, mock_registry):
        data = {'name': 'new_name',
                'uuid': 'input_uuid',
                'parameters': {'username': 'user_a'}}
        db_obj = fakes.FakePlan()
        param_obj = fakes.FakeParameter()
        mock_registry.Plan.return_value = db_obj
        mock_registry.Parameter.return_value = param_obj
        handler = plan_handler.PlanHandler(self.ctx)
        res = handler.create(data)
        db_obj.create.assert_called_once_with(self.ctx)
        param_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)

    @mock.patch('solum.deployer.api.API.destroy_app')
    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_plan_delete(self, mock_kc, mock_destroy, mock_registry):
        db_obj = fakes.FakePlan()
        mock_registry.Plan.get_by_uuid.return_value = db_obj
        handler = plan_handler.PlanHandler(self.ctx)
        handler.delete('test_id')
        mock_destroy.assert_called_once_with(app_id=db_obj.id)
        mock_registry.Plan.get_by_uuid.assert_called_once_with(self.ctx,
                                                               'test_id')

    @mock.patch('solum.deployer.api.API.destroy_app')
    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_plan_delete_with_param(self, mock_kc, mock_destroy,
                                    mock_registry):
        db_obj = fakes.FakePlan()
        param_obj = fakes.FakeParameter()
        mock_registry.Plan.get_by_uuid.return_value = db_obj
        mock_registry.Parameter.get_by_plan_id.return_value = param_obj
        handler = plan_handler.PlanHandler(self.ctx)
        handler.delete('test_id')
        param_obj.destroy.assert_called_once_with(self.ctx)
        mock_registry.Plan.get_by_uuid.assert_called_once_with(self.ctx,
                                                               'test_id')
        mock_registry.Parameter.get_by_plan_id.assert_called_once_with(
            self.ctx, db_obj.id)
        mock_destroy.assert_called_once_with(app_id=db_obj.id)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_trigger_workflow_stage_select(self, mock_kc, mock_registry):
        trigger_id = 1
        plan_obj = fakes.FakePlan()
        artifacts = [{"name": "Test",
                      "artifact_type": "heroku",
                      "content": {"href": "https://github.com/some/project"},
                      "language_pack": "auto"}]
        plan_obj.raw_content = {"artifacts": artifacts}
        mock_registry.Plan.get_by_trigger_id.return_value = plan_obj
        handler = plan_handler.PlanHandler(self.ctx)
        handler._build_artifact = mock.MagicMock()
        handler.trigger_workflow(trigger_id, workflow=['unittest'])
        handler._build_artifact.assert_called_once_with(
            plan_obj,
            artifact=artifacts[0],
            commit_sha='',
            status_url=None,
            workflow=['unittest'],
            )
        mock_registry.Plan.get_by_trigger_id.assert_called_once_with(
            None, trigger_id)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_trigger_workflow_verify_artifact_failed(self, mock_kc,
                                                     mock_registry):

        trigger_id = 1
        plan_obj = fakes.FakePlan()
        artifacts = [{"name": "Test",
                      "artifact_type": "heroku",
                      "content": {"href": "https://github.com/some/project"},
                      "language_pack": "auto"}]
        plan_obj.raw_content = {"artifacts": artifacts}
        mock_registry.Plan.get_by_trigger_id.return_value = plan_obj
        handler = plan_handler.PlanHandler(self.ctx)
        handler._build_artifact = mock.MagicMock()
        handler._verify_artifact = mock.MagicMock(return_value=False)
        collab_url = 'https://api.github.com/repos/u/r/collaborators/foo'
        handler.trigger_workflow(trigger_id, collab_url=collab_url)
        assert not handler._build_artifact.called



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add external_ref to image table

Revision ID: 2c058b5051e4
Revises: 27dff58cbc65
Create Date: 2015-01-05 20:07:41.936667

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '2c058b5051e4'
down_revision = '27dff58cbc65'


def upgrade():
    op.add_column('image',
                  sa.Column('external_ref', sa.String(length=1024)))


def downgrade():
    op.drop_column('image', 'external_ref')



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp import camp_v1_1_endpoint
from solum.api.controllers.camp import platform_endpoints
from solum.api.controllers.camp.v1_1 import root as v1_1_root
from solum.common import exception


class Controller(rest.RestController):
    """CAMP API controller root."""

    platform_endpoints = platform_endpoints.PlatformEndpointsController()
    camp_v1_1_endpoint = camp_v1_1_endpoint.CAMPv11EndpointController()
    v1_1 = v1_1_root.Controller()

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(wtypes.text)
    def get(self):
        return "CAMP be here"



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tempfile

import fixtures
from oslo_config import cfg
from oslo_db import options

from solum.common import context
from solum import objects
from solum.objects.sqlalchemy import models

CONF = cfg.CONF


def dummy_context(user='test_username', tenant_id='fake_project_id',
                  user_name='usr_name'):
    return context.RequestContext(user=user,
                                  project_id=tenant_id,
                                  user_name=user_name)


class Database(fixtures.Fixture):

    def __init__(self):
        super(Database, self).__init__()
        self.db_file = None
        with tempfile.NamedTemporaryFile(suffix='.sqlite',
                                         delete=False) as test_file:
            # note the temp file gets deleted by the NestedTempfile fixture.
            self.db_file = test_file.name
        objects.IMPL.cleanup()

    def setUp(self):
        super(Database, self).setUp()
        self.configure()
        self.addCleanup(objects.IMPL.cleanup)
        models.Base.metadata.create_all(objects.IMPL.get_engine())
        objects.IMPL.get_engine().connect()
        objects.load()

    def configure(self):
        options.cfg.set_defaults(options.database_opts,
                                 sqlite_synchronous=False)
        options.set_defaults(cfg.CONF,
                             connection='sqlite:///%s' % self.db_file)


def get_dummy_session():
    return objects.IMPL.get_session()


def create_models_from_data(model_cls, data, ctx):
    for d in data:
        mdl = model_cls()
        for key, value in d.items():
            setattr(mdl, key, value)
        mdl.create(ctx)
        d['id'] = mdl.id



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import os

from solum.api.controllers.camp.v1_1.datamodel import type_definitions as model
from solum.api.handlers import handler


TYPE_DEFS = {}


class TypeDefinitionHandler(handler.Handler):
    @property
    def proj_dir(self):
        return os.path.normpath(os.path.join(os.path.dirname(__file__),
                                             '..', '..', '..', '..'))

    def _load_typedefs(self):
        typedef_dir_path = os.path.join(self.proj_dir, 'etc', 'solum', 'camp',
                                        'typeDefs')
        for path in os.listdir(typedef_dir_path):
            json_d = json.load(open(os.path.join(typedef_dir_path, path), 'r'))

            # make sure this looks like a type_definition before we
            # deserialize it
            if 'type' in json_d and json_d['type'] == 'type_definition':
                type_d = model.TypeDefinition.from_json(json_d)
                TYPE_DEFS[type_d.uri] = type_d

    def __init__(self, context):
        super(TypeDefinitionHandler, self).__init__(context)
        if len(TYPE_DEFS) == 0:
            self._load_typedefs()

    def get(self, path):
        if path in TYPE_DEFS:
            return TYPE_DEFS[path]

    def get_all(self):
        return TYPE_DEFS.values()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import fixture as config

from solum.api import auth
from solum.api.handlers import assembly_handler
from solum.common import exception
from solum.common import repo_utils
from solum.objects import assembly
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


STATES = assembly.States


@mock.patch('solum.objects.registry')
class TestAssemblyHandler(base.BaseTestCase):
    def setUp(self):
        super(TestAssemblyHandler, self).setUp()
        self.ctx = utils.dummy_context()
        self.CONF = self.useFixture(config.Config())
        self.CONF.config(www_authenticate_uri='http://fakeidentity.com',
                         group=auth.OPT_GROUP_NAME)
        self.CONF.config(keystone_version='3')

    def test_assembly_get(self, mock_registry):
        mock_registry.return_value.Assembly.get_by_uuid.return_value = {
            'plan_id': '1234'
        }
        handler = assembly_handler.AssemblyHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        get_by_uuid = mock_registry.Assembly.get_by_uuid
        get_by_uuid.assert_called_once_with(self.ctx, 'test_id')

    def test_assembly_get_all(self, mock_registry):
        mock_registry.AssemblyList.get_all.return_value = {}
        handler = assembly_handler.AssemblyHandler(self.ctx)
        res = handler.get_all()
        self.assertIsNotNone(res)
        mock_registry.AssemblyList.get_all.assert_called_once_with(self.ctx)

    def test_update(self, mock_registry):
        data = {'user_id': 'new_user_id',
                'plan_uuid': 'input_plan_uuid'}
        handler = assembly_handler.AssemblyHandler(self.ctx)
        handler.update('test_id', data)
        mock_registry.Assembly.update_and_save.assert_called_once_with(
            self.ctx, 'test_id', data)

    @mock.patch('solum.worker.api.API.build_app')
    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_create(self, mock_kc, mock_pa, mock_registry):
        data = {'user_id': 'new_user_id',
                'uuid': 'input_uuid',
                'plan_uuid': 'input_plan_uuid'}

        db_obj = fakes.FakeAssembly()
        mock_registry.Assembly.return_value = db_obj
        fp = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = fp
        fp.raw_content = {
            'name': 'theplan',
            'artifacts': [{'name': 'nodeus',
                           'artifact_type': 'heroku',
                           'content': {'private': False,
                                       'href': 'https://example.com/ex.git'},
                           'language_pack': 'auto'}]}

        mock_registry.Image.return_value = fakes.FakeImage()

        handler = assembly_handler.AssemblyHandler(self.ctx)
        res = handler.create(data)
        db_obj.update.assert_called_once_with(data)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)
        git_info = {
            'source_url': "https://example.com/ex.git",
            'commit_sha': '',
            'repo_token': None,
            'status_url': None,
        }
        mock_pa.assert_called_once_with(
            verb='launch_workflow', workflow=['unittest', 'build', 'deploy'],
            build_id=8, name='nodeus', assembly_id=8,
            git_info=git_info, test_cmd=None, ports=[80],
            base_image_id='auto', source_format='heroku',
            image_format='qcow2', run_cmd=None)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_create_with_username_in_ctx(self, mock_kc, mock_registry):
        data = {'plan_uuid': 'input_plan_uuid'}

        db_obj = fakes.FakeAssembly()
        mock_registry.Assembly.return_value = db_obj
        fp = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = fp
        fp.raw_content = {'name': 'theplan'}

        handler = assembly_handler.AssemblyHandler(self.ctx)
        res = handler.create(data)

        self.assertEqual(res.username, self.ctx.user_name)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_create_without_username_in_ctx(self, mock_kc, mock_registry):
        data = {'plan_uuid': 'input_plan_uuid'}

        ctx = utils.dummy_context()
        ctx.user_name = ''
        db_obj = fakes.FakeAssembly()
        mock_registry.Assembly.return_value = db_obj
        fp = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = fp
        fp.raw_content = {'name': 'theplan'}

        handler = assembly_handler.AssemblyHandler(ctx)
        res = handler.create(data)

        self.assertEqual(res.username, '')

    @mock.patch('solum.worker.api.API.build_app')
    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    def test_create_with_private_github_repo(self, mock_kc, mock_pa,
                                             mock_registry):
        data = {'user_id': 'new_user_id',
                'uuid': 'input_uuid',
                'plan_uuid': 'input_plan_uuid'}

        db_obj = fakes.FakeAssembly()
        mock_registry.Assembly.return_value = db_obj
        fp = fakes.FakePlan()
        mock_registry.Plan.get_by_id.return_value = fp
        fp.raw_content = {
            'name': 'theplan',
            'artifacts': [{'name': 'nodeus',
                           'artifact_type': 'heroku',
                           'content': {'private': True,
                                       'href': 'https://example.com/ex.git',
                                       'public_key': 'ssh-rsa abc'},
                           'language_pack': 'auto'}]}
        fp.deploy_keys_uri = 'secret_ref_uri'
        mock_registry.Image.return_value = fakes.FakeImage()

        handler = assembly_handler.AssemblyHandler(self.ctx)
        res = handler.create(data)
        db_obj.update.assert_called_once_with(data)
        db_obj.create.assert_called_once_with(self.ctx)
        self.assertEqual(db_obj, res)
        git_info = {
            'source_url': "https://example.com/ex.git",
            'commit_sha': '',
            'repo_token': None,
            'status_url': None,
        }
        mock_pa.assert_called_once_with(
            verb='launch_workflow', workflow=['unittest', 'build', 'deploy'],
            build_id=8, name='nodeus', assembly_id=8,
            git_info=git_info, ports=[80],
            test_cmd=None, base_image_id='auto', source_format='heroku',
            image_format='qcow2', run_cmd=None)

    @mock.patch('solum.common.clients.OpenStackClients.keystone')
    @mock.patch('solum.deployer.api.API.destroy_assembly')
    @mock.patch('solum.conductor.api.API.update_assembly')
    def test_delete(self, mock_cond, mock_deploy, mock_kc, mock_registry):
        db_obj = fakes.FakeAssembly()
        mock_registry.Assembly.get_by_uuid.return_value = db_obj
        handler = assembly_handler.AssemblyHandler(self.ctx)
        handler.delete('test_id')
        mock_registry.Assembly.get_by_uuid.assert_called_once_with(self.ctx,
                                                                   'test_id')
        mock_cond.assert_called_once_with(db_obj.id, {'status': 'DELETING'})
        mock_deploy.assert_called_once_with(assem_id=db_obj.id)

    @mock.patch('httplib2.Http.request')
    def test_verify_artifact_raise_exp(self, http_mock, mock_registry):
        artifact = {"name": "Test",
                    "artifact_type": "heroku",
                    "content": {"href": "https://github.com/some/project"},
                    "language_pack": "auto",
                    "repo_token": "abcd"}
        http_mock.return_value = ({'status': '404'}, '')  # Not a collaborator
        collab_url = 'https://api.github.com/repos/u/r/collaborators/foo'
        self.assertRaises(exception.RequestForbidden,
                          repo_utils.verify_artifact,
                          artifact, collab_url)



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum Conductor default handler."""

from oslo_log import log as logging
from sqlalchemy import exc as sqla_exc

from solum import objects
from solum.objects import assembly

LOG = logging.getLogger(__name__)

ASSEMBLY_STATES = assembly.States


class Handler(object):
    def __init__(self):
        super(Handler, self).__init__()
        objects.load()

    def echo(self, ctxt, message):
        LOG.debug("%s" % message)

    def build_job_update(self, ctxt, build_id, status, description,
                         created_image_id, docker_image_name, assembly_id):
        to_update = {'status': status,
                     'external_ref': created_image_id,
                     'docker_image_name': docker_image_name,
                     'description': str(description)}
        try:
            objects.registry.Image.update_and_save(ctxt, build_id, to_update)
        except sqla_exc.SQLAlchemyError as ex:
            LOG.error("Failed to update image, ID: %s" % build_id)
            LOG.exception(ex)

        # create the component if needed.
        if assembly_id is None:
            return
        try:
            assem = objects.registry.Assembly.get_by_id(ctxt,
                                                        assembly_id)
            if assem.status == ASSEMBLY_STATES.DELETING:
                return
            if not any([comp for comp in assem.components
                        if 'Image_Build' in comp.description]):
                comp_name = "Heat_Stack_for_%s" % assem.name
                stack_id = None
                if assem.heat_stack_component is not None:
                    stack_id = assem.heat_stack_component.heat_stack_id
                objects.registry.Component.assign_and_create(ctxt, assem,
                                                             comp_name,
                                                             'Image_Build',
                                                             'Image Build job',
                                                             created_image_id,
                                                             stack_id)
                # update reference to image in assembly
                assem_update = {'image_id': build_id}
                objects.registry.Assembly.update_and_save(ctxt,
                                                          assembly_id,
                                                          assem_update)
        except sqla_exc.IntegrityError:
            LOG.error("IntegrityError in creating Image_Build component,"
                      " assembly %s may be deleted" % assembly_id)

    def update_assembly(self, ctxt, assembly_id, data):
        try:
            objects.registry.Assembly.update_and_save(ctxt, assembly_id, data)
        except sqla_exc.SQLAlchemyError as ex:
            LOG.error("Failed to update assembly status, ID: %s" % assembly_id)
            LOG.exception(ex)

    def update_image(self, ctxt, image_id, status, external_ref=None,
                     docker_image_name=None):
        to_update = {'status': status}
        if external_ref:
            to_update['external_ref'] = external_ref
        if docker_image_name:
            to_update['docker_image_name'] = docker_image_name
        try:
            objects.registry.Image.update_and_save(ctxt, image_id, to_update)
        except sqla_exc.SQLAlchemyError as ex:
            LOG.error("Failed to update image, ID: %s" % image_id)
            LOG.exception(ex)



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Create initial Solum DB schema

Revision ID: 498adc6185ae
Revises: None
Create Date: 2014-02-03 15:51:20.819539

"""
from alembic import op
import sqlalchemy as sa

from solum.objects.sqlalchemy import models

from oslo_utils import timeutils

# revision identifiers, used by Alembic.
revision = '498adc6185ae'
down_revision = None


def upgrade():
    op.create_table(
        'sensor',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('project_id', sa.String(length=36)),
        sa.Column('user_id', sa.String(length=36)),
        sa.Column('name', sa.String(255)),
        sa.Column('sensor_type', sa.String(255)),
        sa.Column('value', sa.String(255)),
        sa.Column('timestamp', sa.DateTime),
        sa.Column('description', sa.String(255)),
        sa.Column('documentation', sa.String(255)),
        sa.Column('target_resource', sa.String(255)),
    )

    op.create_table(
        'operation',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('tags', sa.Text),
        sa.Column('documentation', sa.Text),
        sa.Column('target_resource', sa.Text)
    )

    op.create_table(
        'image',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('source_uri', sa.String(1024)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('tags', sa.Text),
        sa.Column('state', sa.String(12)),
        sa.Column('base_image_id', sa.String(length=36)),
        sa.Column('created_image_id', sa.String(length=36)),
        sa.Column('image_format', sa.String(length=12)),
        sa.Column('source_format', sa.String(length=36)),
    )

    op.create_table(
        'extension',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('description', sa.String(255)),
        sa.Column('name', sa.String(100)),
        sa.Column('version', sa.String(16)),
        sa.Column('documentation', sa.String(255)),
        # NOTE(stannie): tags will be added in a dedicated table
    )

    op.create_table(
        'plan',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('project_id', sa.String(length=36)),
        sa.Column('user_id', sa.String(length=36)),
        sa.Column('raw_content', models.YAMLEncodedDict(2048)),
        sa.Column('description', sa.String(length=255)),
        sa.Column('name', sa.String(255)),
        sa.Column('deploy_keys_uri', sa.String(length=1024)),
    )

    op.create_table(
        'assembly',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('tags', sa.Text),
        sa.Column('plan_id', sa.Integer, sa.ForeignKey('plan.id'),
                  nullable=False),
        sa.Column('status', sa.String(length=36)),
        sa.Column('trigger_id', sa.String(length=36)),
        sa.Column('trust_id', sa.String(length=255)),
        sa.Column('application_uri', sa.String(length=1024)),
    )

    op.create_table(
        'pipeline',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('tags', sa.Text),
        sa.Column('plan_id', sa.Integer, sa.ForeignKey('plan.id'),
                  nullable=False),
        sa.Column('workbook_name', sa.String(length=255)),
        sa.Column('trigger_id', sa.String(length=36)),
        sa.Column('trust_id', sa.String(length=255)),
    )

    op.create_table(
        'execution',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('pipeline_id', sa.Integer, sa.ForeignKey('pipeline.id')),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
    )

    op.create_table(
        'infrastructure_stack',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('tags', sa.Text),
        sa.Column('image_id', sa.String(36), nullable=False),
        sa.Column('heat_stack_id', sa.String(36)),
    )

    op.create_table(
        'component',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('component_type', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('tags', sa.Text),
        sa.Column('assembly_id', sa.Integer, sa.ForeignKey('assembly.id')),
        sa.Column('parent_component_id', sa.Integer,
                  sa.ForeignKey('component.id')),
        sa.Column('resource_uri', sa.String(length=1024)),
        sa.Column('heat_stack_id', sa.String(36)),
    )

    op.create_table(
        'service',
        sa.Column('id', sa.Integer, primary_key=True, nullable=False),
        sa.Column('uuid', sa.String(36), nullable=False),
        sa.Column('created_at', sa.DateTime, default=timeutils.utcnow),
        sa.Column('updated_at', sa.DateTime, onupdate=timeutils.utcnow),
        sa.Column('name', sa.String(100)),
        sa.Column('description', sa.String(255)),
        sa.Column('project_id', sa.String(36)),
        sa.Column('user_id', sa.String(36)),
        sa.Column('service_type', sa.String(100)),
        sa.Column('read_only', sa.Integer, default=0),
        sa.Column('tags', sa.Text),
    )


def downgrade():
    op.drop_table('service')
    op.drop_table('component')
    op.drop_table('assembly')
    op.drop_table('plan')
    op.drop_table('extension')
    op.drop_table('image')
    op.drop_table('operation')
    op.drop_table('sensor')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
import requests
from requests import exceptions
import urllib

from solum.i18n import _

LOG = logging.getLogger(__name__)


def get(url, max_size, chunk_size=None, allowed_schemes=('http', 'https')):
    """Get the data at the specified URL.

    The URL must use the http: or https: schemes.
    The file: scheme is also supported if you override
    the allowed_schemes argument.
    The max_size represents the total max byte of your file.
    The chunk_size is by default set at max_size, it represents the size
    of your chunk.
    Raise an IOError if getting the data fails.
    Raise an IOError if max_size is less than 1.
    Raise an IOError if chunk_size is less than 1.
    """

    LOG.info(_('Fetching data from %s') % url)

    components = urllib.parse.urlparse(url)

    if components.scheme not in allowed_schemes:
        raise IOError(_('Invalid URL scheme %s') % components.scheme)

    if chunk_size is None:
        chunk_size = max_size
    if max_size < 1:
        raise IOError("max_size should be greater than 0")
    if chunk_size < 1:
        raise IOError("chunk_size should be greater than 0")

    if components.scheme == 'file':
        try:
            return urllib.request.urlopen(url).read()
        except urllib.error.URLError as uex:
            raise IOError(_('Failed to read file: %s') % str(uex))

    try:
        resp = requests.get(url, stream=True)
        resp.raise_for_status()

        # We cannot use resp.text here because it would download the
        # entire file, and a large enough file would bring down the
        # engine.  The 'Content-Length' header could be faked, so it's
        # necessary to download the content in chunks to until
        # max_size is reached.  The chunk_size we use needs
        # to balance CPU-intensive string concatenation with accuracy
        # (eg. it's possible to fetch 1000 bytes greater than
        # max_size with a chunk_size of 1000).
        reader = resp.iter_content(chunk_size=chunk_size)
        result = ""
        for chunk in reader:
            result += chunk
            if len(result) > max_size:
                raise IOError("File exceeds maximum allowed size (%s "
                              "bytes)" % max_size)
        return result

    except exceptions.RequestException as ex:
        raise IOError(_('Failed to retrieve file: %s') % str(ex))



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""API for interfacing with Solum Worker."""

from oslo_config import cfg

from solum.common.rpc import service


class API(service.API):
    def __init__(self, context=None):
        cfg.CONF.import_opt('topic', 'solum.worker.config',
                            group='worker')
        super(API, self).__init__(context,
                                  topic=cfg.CONF.worker.topic)

    def build_app(self, verb, build_id, git_info, ports, name, base_image_id,
                  source_format, image_format, assembly_id, workflow,
                  test_cmd=None, run_cmd=None, du_id=None):
        self._cast(verb, build_id=build_id, git_info=git_info, ports=ports,
                   name=name, base_image_id=base_image_id,
                   source_format=source_format, image_format=image_format,
                   assembly_id=assembly_id, workflow=workflow,
                   test_cmd=test_cmd, run_cmd=run_cmd, du_id=du_id)

    def build_lp(self, image_id, git_info, name, source_format, image_format,
                 artifact_type, lp_params=None):
        self._cast('build_lp', image_id=image_id, git_info=git_info, name=name,
                   source_format=source_format, image_format=image_format,
                   artifact_type=artifact_type, lp_params=lp_params)



# Copyright 2015 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.objects import registry
from solum.objects.sqlalchemy import workflow
from solum.tests import base
from solum.tests import utils


class TestWorkflow(base.BaseTestCase):
    def setUp(self):
        super(TestWorkflow, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()
        self.data = [self._get_data(self.ctx.project_id)]
        utils.create_models_from_data(workflow.Workflow, self.data, self.ctx)

    def _get_data(self, project_id):

        source = dict()
        source['repository'] = "https://github.com"
        source['revision'] = "master"

        workflow = dict()
        workflow["test_cmd"] = "./unit_tests.sh"
        workflow["run_cmd"] = "python app.py"

        jsondata = dict()
        jsondata['source'] = source
        jsondata['config'] = workflow
        jsondata['actions'] = ["test", "build", "deploy"]
        jsondata['project_id'] = project_id
        jsondata['user_id'] = 'fred'
        jsondata['id'] = '123'
        return jsondata

    def test_objects_registered(self):
        self.assertTrue(registry.Workflow)
        self.assertTrue(registry.WorkflowList)

    def test_get_all(self):
        lst = workflow.WorkflowList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data_by_id(self):
        foundwf = workflow.Workflow().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(foundwf, key))

    def test_check_data_by_uuid(self):
        foundwf = workflow.Workflow().get_by_uuid(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(foundwf, key))



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from mistralclient.api import base
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils

from solum.api.handlers import handler
from solum.common import catalog
from solum.common import clients
from solum.common import context
from solum.common import exception
from solum.common import heat_utils
from solum.common import yamlutils
from solum import objects

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class PipelineHandler(handler.Handler):
    """Fulfills a request on the pipeline resource."""
    def __init__(self, context):
        super(PipelineHandler, self).__init__(context)
        self._clients = None
        if context is not None:
            self._clients = clients.OpenStackClients(context)

    def get(self, id):
        """Return an pipeline."""
        return objects.registry.Pipeline.get_by_uuid(self.context, id)

    def _context_from_trust_id(self, trust_id):
        cntx = context.RequestContext(trust_id=trust_id)
        self._clients = clients.OpenStackClients(cntx)
        return self._clients.keystone().context

    def trigger_workflow(self, trigger_id):
        """Get trigger by trigger id and execute the associated workbook."""
        # Note: self.context will be None at this point as this is a
        # non-authenticated request.
        db_obj = objects.registry.Pipeline.get_by_trigger_id(None,
                                                             trigger_id)
        # get the trust context and authenticate it.
        try:
            self.context = self._context_from_trust_id(db_obj.trust_id)
        except exception.AuthorizationFailure as auth_ex:
            LOG.warning(auth_ex)
            return

        self._execute_workbook(db_obj)

    def _get_context_from_last_execution(self, pipeline):

        last_execution = pipeline.last_execution()
        if last_execution is None:
            return

        osc = self._clients
        try:
            execution = osc.mistral().executions.get(
                pipeline.workbook_name, last_execution.uuid)
            execution_ctx = json.loads(execution.context)
            str_definition = osc.mistral().workbooks.get_definition(
                pipeline.workbook_name)
            definition = yamlutils.load(str_definition)
        except base.APIException:
            LOG.debug('Could not get last_execution(%s)' %
                      last_execution, exc_info=True)
            return

        tasks = definition['Workflow'].get('tasks', {})
        inputs = set()
        outputs = set()
        for act in tasks:
            inputs |= set(tasks[act].get('parameters', {}).keys())
            outputs |= set(tasks[act].get('publish', {}).keys())
        inputs -= outputs
        return dict((key, execution_ctx.get(key)) for key in inputs)

    def _build_execution_context(self, pipeline):
        # try and read the context from the previous execution
        ctx = self._get_context_from_last_execution(pipeline)
        if ctx is not None:
            return ctx

        ctx = {}
        # service urls.
        kc = self._clients.keystone()
        ctx['heat_service_url'] = kc.client.service_catalog.url_for(
            service_type='orchestration',
            interface='publicURL')
        ctx['build_service_url'] = kc.client.service_catalog.url_for(
            service_type='image_builder',
            interface='publicURL')

        # extract context from the plan
        # TODO(asalkeld) this should be versioned.
        plan_obj = objects.registry.Plan.get_by_id(self.context,
                                                   pipeline.plan_id)
        ctx['name'] = plan_obj.name

        artifacts = plan_obj.raw_content.get('artifacts', [])
        for arti in artifacts:
            ctx['source_uri'] = arti['content']['href']
            ctx['base_image_id'] = arti.get('language_pack', 'auto')
            ctx['source_format'] = arti.get('artifact_type', 'heroku')
            ctx['image_format'] = arti.get('image_format',
                                           CONF.api.image_format)

        ctx['template'] = catalog.get('templates', 'basic')
        # TODO(asalkeld) add support to the plan to pass heat parameters.
        ctx['parameters'] = {'app_name': pipeline.name}
        ctx['parameters'].update(
            heat_utils.get_network_parameters(self._clients))
        ctx['stack_id'] = self._create_empty_stack(pipeline)
        ctx['stack_name'] = pipeline.name

        # TODO(asalkeld) integrate the Environment into the context.
        return ctx

    def _create_empty_stack(self, pipeline):
        osc = self._clients
        template = catalog.get('templates', 'empty')
        created_stack = osc.heat().stacks.create(stack_name=pipeline.name,
                                                 template=template)
        return created_stack['stack']['id']

    def _delete_stack(self, pipeline):
        execution_ctx = self._get_context_from_last_execution(pipeline)
        if execution_ctx is not None and 'stack_id' in execution_ctx:
            osc = self._clients
            osc.heat().stacks.delete(stack_id=execution_ctx['stack_id'])

    def _execute_workbook(self, pipeline):
        execution_ctx = self._build_execution_context(pipeline)

        osc = self._clients
        resp = osc.mistral().executions.create(pipeline.workbook_name,
                                               'start',
                                               execution_ctx)
        ex_obj = objects.registry.Execution()
        ex_obj.uuid = resp.id
        ex_obj.pipeline_id = pipeline.id
        ex_obj.create(self.context)

    def _ensure_workbook(self, pipeline):
        osc = clients.OpenStackClients(self.context)
        try:
            osc.mistral().workbooks.get(pipeline.workbook_name)
        except Exception as excp:
            if 'Workbook not found' in str(excp):
                definition = catalog.get('workbooks', pipeline.workbook_name)
                # create the workbook for the user.
                osc.mistral().workbooks.create(
                    pipeline.workbook_name,
                    'solum generated workbook',
                    ['solum', 'builtin'])
                osc.mistral().workbooks.upload_definition(
                    pipeline.workbook_name,
                    definition)
            else:
                raise

    def update(self, id, data):
        """Modify a resource."""
        updated = objects.registry.Pipeline.update_and_save(self.context,
                                                            id, data)
        return updated

    def delete(self, id):
        """Delete a resource."""
        db_obj = objects.registry.Pipeline.get_by_uuid(self.context, id)

        self._delete_stack(db_obj)

        # delete the trust.
        self._clients.keystone().delete_trust(db_obj.trust_id)
        db_obj.destroy(self.context)

    def create(self, data):
        """Create a new resource."""
        db_obj = objects.registry.Pipeline()
        db_obj.update(data)
        db_obj.uuid = uuidutils.generate_uuid()
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.project_id
        db_obj.trigger_id = uuidutils.generate_uuid()

        # create the trust_id and store it.
        trust_context = self._clients.keystone().create_trust_context()
        db_obj.trust_id = trust_context.trust_id
        db_obj.create(self.context)

        self._ensure_workbook(db_obj)
        self._execute_workbook(db_obj)

        return db_obj

    def get_all(self):
        """Return all pipelines, based on the query provided."""
        return objects.registry.PipelineList.get_all(self.context)



# Copyright 2014 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os.path
from unittest import mock


from solum.common import catalog
from solum.common import exception
from solum.tests import base


class TestCatalog(base.BaseTestCase):
    def setUp(self):
        super(TestCatalog, self).setUp()
        self.proj_dir = catalog.CONF.get('source_path')
        if not self.proj_dir:
            self.proj_dir = os.path.realpath(
                os.path.join(os.path.dirname(__file__),
                             '..', '..', '..'))

    def test_get_default(self):
        with mock.patch('solum.common.catalog.open',
                        mock.mock_open(read_data='test content'),
                        create=True) as m_open:
            val = catalog.get('test', 'test_data')
            self.assertEqual('test content', val)
            expect = os.path.join(self.proj_dir,
                                  'etc/solum/test/test_data.yaml')
            m_open.assert_called_with(expect)

    def test_get_content_type(self):
        with mock.patch('solum.common.catalog.open',
                        mock.mock_open(read_data='test content'),
                        create=True) as m_open:
            val = catalog.get('test', 'test_data', content_type='fake')
            self.assertEqual('test content', val)
            expect = os.path.join(self.proj_dir,
                                  'etc/solum/test/test_data.fake')
            m_open.assert_called_with(expect)

    def test_get_fail(self):
        with mock.patch('solum.common.catalog.open',
                        mock.mock_open(read_data='test content'),
                        create=True) as m_open:
            m_open.side_effect = IOError('test')
            self.assertRaises(exception.ObjectNotFound,
                              catalog.get, 'test', 'test_data')



# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_config import cfg
from oslo_log import log as logging

from solum.common import exception as exc
from solum.common import solum_swiftclient as swiftclient
import solum.uploaders.common

from swiftclient import exceptions as swiftexp

LOG = logging.getLogger(__name__)

cfg.CONF.import_opt('log_upload_swift_container', 'solum.worker.config',
                    group='worker')


class SwiftUpload(solum.uploaders.common.UploaderBase):
    strategy = "swift"

    def _upload(self, container, filename, filepath):
        swift = swiftclient.SwiftClient(self.context)
        swift.upload(filepath, container, filename)

    def upload_log(self):
        container = cfg.CONF.worker.log_upload_swift_container
        filename = "%s-%s/%s-%s.log" % (self.resource.name, self.stage_id,
                                        self.stage_name, self.stage_id)

        self.transform_jsonlog()
        try:
            LOG.debug("Uploading log to Swift. %s, %s" %
                      (container, filename))
            self._upload(container, filename, self.transformed_path)
        except exc.InvalidObjectSizeError:
            LOG.exception("Unable to upload logfile: %s to swift. "
                          "Invalid size." % self.transformed_path)
            return
        except swiftexp.ClientException:
            LOG.exception("Failed to upload logfile: %s to Swift." %
                          self.transformed_path)
            return
        LOG.debug("Logfile uploaded to Swift.")

        swift_info = {
            'container': container,
        }

        self.write_userlog_row(filename, swift_info)



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_utils import uuidutils
import wsme
from wsme.rest import json as wjson
from wsme import types as wtypes

from solum.api.controllers.v1.datamodel import types as api_types


class Requirement(wtypes.Base):

    requirement_type = wtypes.text
    "Type of requirement."

    fulfillment = wtypes.text
    "The ID of the service."


class ServiceReference(wtypes.Base):

    name = wtypes.text
    "The name of the service to be used."

    id = wtypes.text
    "The ID of the service to be used."

    characteristics = [wtypes.text]
    "A list of characteristics required from the service."


class Artifact(wtypes.Base):
    """Artifact."""

    name = wtypes.text
    "Name of the artifact."

    artifact_type = wtypes.text
    "Type of artifact."

    content = {wtypes.text: api_types.MultiType(wtypes.text, bool)}
    "Type specific content as a flat dict."

    ports = api_types.MultiType(api_types.PortType, [api_types.PortType],
                                {wtypes.text: api_types.PortType})
    "Type of ports which an app listens on"

    language_pack = wtypes.text
    "The Language pack required for this artifact."

    requirements = [Requirement]
    "List of requirements for the artifact."

    unittest_cmd = wtypes.text
    "Optional unit test command for the artifact."

    run_cmd = wtypes.text
    "The command for starting up user code when a DU is deployed."

    repo_token = wtypes.text
    "The OAuth token to access repository"

    def __init__(self, **kwargs):
        if 'requirements' in kwargs:
            kwargs['requirements'] = [Requirement(**re)
                                      for re in kwargs.get('requirements', [])]
        super(Artifact, self).__init__(**kwargs)


class Plan(api_types.Base):
    """Representation of an Plan file.

    The Plan resource is a representation of a Plan file. Plans are used to
    create Assembly resources. A Plan resource may be used to create an
    arbitrary number of Assembly instances. They use artifacts and services
    to indicate what will be used to generate the plan, and what services
    Solum can use to satisfy them. Note: Plan files are YAML and Plan resources
    are the REST representation of the Plan file after services have been
    matched to ones offered by Solum.
    """

    artifacts = [Artifact]
    """List of artifacts defining the plan."""

    services = [ServiceReference]
    """List of services needed by the plan."""

    trigger_uri = wtypes.text
    """The trigger uri used to trigger the build of the plan"""

    parameters = {wtypes.text: api_types.MultiType(wtypes.text,
                                                   int,
                                                   bool,
                                                   float)}
    """User defined parameters"""

    def __init__(self, **kwargs):
        if 'artifacts' in kwargs:
            kwargs['artifacts'] = [Artifact(**art)
                                   for art in kwargs.get('artifacts', [])]
        if 'services' in kwargs:
            kwargs['services'] = [ServiceReference(**sr)
                                  for sr in kwargs.get('services', [])]
        super(Plan, self).__init__(**kwargs)

    @classmethod
    def sample(cls):
        return cls(uri='http://example.com/v1/plans/x1',
                   name='Example-plan',
                   type='plan',
                   tags=['small'],
                   artifacts=[{
                       'name': 'My-python-app',
                       'artifact_type': 'git_pull',
                       'content': {'href': 'git://example.com/project.git',
                                   'private': False},
                       'language_pack': uuidutils.generate_uuid(),
                       'requirements': [{
                           'requirement_type': 'git_pull',
                           'fulfillment': 'id:build'}]}],
                   services=[{
                       'name': 'Build-Service',
                       'id': 'build',
                       'characteristics': ['python_build_service']}],
                   project_id='1dae5a09ef2b4d8cbf3594b0eb4f6b94',
                   user_id='55f41cf46df74320b9486a35f5d28a11',
                   trigger_uri='http://example.com/v1/triggers/1abc234',
                   description='A plan with no services or artifacts shown')

    @classmethod
    def from_db_model(cls, m, host_url):
        json = m.as_dict()
        json['type'] = m.__tablename__
        json['uri'] = '%s/v1/%s/%s' % (host_url, m.__resource__, m.uuid)
        if m.raw_content is not None:
            json.update(m.raw_content)
        del json['id']
        return cls(**(json))

    def as_dict(self, db_model):
        base = super(Plan, self).as_dict(db_model)
        if self.artifacts is not wsme.Unset:
            base.update({'artifacts': [wjson.tojson(Artifact, art)
                                       for art in self.artifacts]})
        if self.services is not wsme.Unset:
            base.update({'services': [wjson.tojson(ServiceReference, ref)
                                      for ref in self.services]})
        if self.parameters is not wsme.Unset:
            base.update({'parameters': self.parameters})
        return base



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp import camp_v1_1_endpoint as endpoint
from solum.api.controllers.camp.datamodel import platform_endpoints as model
from solum.api.controllers import common_types
from solum.common import exception


URI_STRING = '%s/camp/platform_endpoints'
DESCRIPTION_STRING = "Solum CAMP API platform_endpoints resource."


class PlatformEndpointsController(rest.RestController):
    """platform_endpoints resource controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.PlatformEndpoints)
    def get(self):
        host_url = pecan.request.application_url.rstrip('/')
        links = [
            common_types.Link(href=endpoint.URI_STRING % host_url,
                              target_name=endpoint.NAME_STRING)
        ]

        return model.PlatformEndpoints(uri=URI_STRING % host_url,
                                       name='Solum_CAMP_endpoints',
                                       type='platform_endpoints',
                                       description=DESCRIPTION_STRING,
                                       platform_endpoint_links=links)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import fixture as config

from solum.api import auth
from solum.api.handlers import workflow_handler
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


@mock.patch('solum.objects.registry')
class TestWorkflowHandler(base.BaseTestCase):
    def setUp(self):
        super(TestWorkflowHandler, self).setUp()
        self.ctx = utils.dummy_context()
        self.CONF = self.useFixture(config.Config())
        self.CONF.config(www_authenticate_uri='http://fakeidentity.com',
                         group=auth.OPT_GROUP_NAME)
        self.CONF.config(keystone_version='3')

    def test_workflow_get(self, mock_registry):
        mock_registry.return_value.Workflow.get_by_uuid.return_value = {
            'app_id': '1234'
        }
        handler = workflow_handler.WorkflowHandler(self.ctx)
        res = handler.get('test_id')
        self.assertIsNotNone(res)
        get_by_uuid = mock_registry.Workflow.get_by_uuid
        get_by_uuid.assert_called_once_with(self.ctx, 'test_id')

    def test_workflow_get_all(self, mock_reg):
        mock_reg.WorkflowList.get_all.return_value = {}
        handler = workflow_handler.WorkflowHandler(self.ctx)
        res = handler.get_all(app_id='123')
        self.assertIsNotNone(res)
        mock_reg.WorkflowList.get_all.assert_called_once_with(self.ctx,
                                                              app_id='123')

    def test_delete(self, mock_registry):
        db_obj = fakes.FakeWorkflow()
        mock_registry.Workflow.get_by_uuid.return_value = db_obj
        handler = workflow_handler.WorkflowHandler(self.ctx)
        handler.delete('test_id')
        mock_registry.Workflow.get_by_uuid.assert_called_once_with(self.ctx,
                                                                   'test_id')

    @mock.patch('solum.worker.api.API.build_app')
    @mock.patch('solum.objects.sqlalchemy.workflow.Workflow.insert')
    def test_create(self, mock_wf_insert, mock_pa, mock_registry):

        app_obj = fakes.FakeApp()
        app_id = app_obj.id
        test_cmd = app_obj.workflow_config['test_cmd']
        run_cmd = app_obj.workflow_config['run_cmd']
        mock_registry.App.get_by_id.return_value = app_obj

        workflow_data = {"app_id": app_id,
                         "source": app_obj.source,
                         "config": app_obj.workflow_config,
                         "actions": app_obj.trigger_actions}

        fp = fakes.FakePlan()
        mock_registry.Plan.return_value = fp

        fa = fakes.FakeAssembly()
        fa.plan_uuid = fp.uuid
        mock_registry.Assembly.return_value = fa

        wf_obj = fakes.FakeWorkflow()
        wf_obj.app_id = app_obj.id
        wf_obj.assembly = fa.id
        mock_registry.Workflow.return_value = wf_obj

        fi = fakes.FakeImage()
        mock_registry.Image.return_value = fi

        handler = workflow_handler.WorkflowHandler(self.ctx)

        res = handler.create(workflow_data, commit_sha='master', status_url='',
                             du_id='')
        self.assertEqual(wf_obj, res)
        git_info = {
            'source_url': app_obj.source['repository'],
            'commit_sha': app_obj.source['revision'],
            'repo_token': app_obj.source['repo_token'],
            'private': app_obj.source['private'],
            'private_ssh_key': app_obj.source['private_ssh_key'],
            'status_url': '',
        }
        mock_pa.assert_called_once_with(
            verb='launch_workflow', workflow=['unittest', 'build', 'deploy'],
            build_id=fa.id, name=fi.name, assembly_id=fa.id,
            git_info=git_info, test_cmd=test_cmd, ports=app_obj.ports,
            base_image_id=fi.base_image_id,
            source_format=fi.source_format,
            image_format=fi.image_format, run_cmd=run_cmd, du_id='')



# Copyright 2014 - Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.api.controllers import common_types


class Execution(common_types.Link):
    @classmethod
    def from_db_model(cls, m, host_url):
        json = {'target_name': m.uuid}
        json['href'] = '%s/v1/%s/%s' % (host_url, m.__resource__, m.uuid)
        return cls(**(json))

    def as_dict(self, db_model):
        return {'uuid': self.target_name}



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
from unittest import mock


from solum.cmd import db_manage as cli
from solum.tests import base


class TestCli(base.BaseTestCase):
    scenarios = [
        ('stamp', dict(
            argv=['prog', 'stamp', 'foo'],
            func_name='stamp',
            exp_args=('foo',), exp_kwargs={})),
        ('version', dict(
            argv=['prog', 'version'],
            func_name='version',
            exp_args=[], exp_kwargs={})),
        ('revision_auto', dict(
            argv=['prog', 'revision', '--autogenerate', '-m' 'message'],
            func_name='revision', exp_args=[],
            exp_kwargs={'message': 'message',
                        'autogenerate': True})),
        ('revision', dict(
            argv=['prog', 'revision', '-m', 'message'],
            func_name='revision', exp_args=[],
            exp_kwargs={'message': 'message',
                        'autogenerate': False})),
        ('upgrade', dict(
            argv=['prog', 'upgrade', 'head'],
            func_name='upgrade', exp_args=('head',),
            exp_kwargs={})),
        ('downgrade', dict(
            argv=['prog', 'downgrade', 'folsom'],
            func_name='downgrade', exp_args=('folsom',),
            exp_kwargs={})),
    ]

    def setUp(self):
        super(TestCli, self).setUp()
        self.addCleanup(cli.CONF.reset)

    @mock.patch.object(cli, 'get_manager')
    def test_alembic_manager(self, mock_get_manager):
        manager = mock_get_manager.return_value
        with mock.patch.object(sys, 'argv', self.argv):
            cli.main()
            mock_get_manager.assert_called_once_with()
            test_method = getattr(manager, self.func_name)
            test_method.assert_called_once_with(*self.exp_args,
                                                **self.exp_kwargs)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlalchemy

from solum.objects import sensor as abstract
from solum.objects.sqlalchemy import models as sql


class Sensor(sql.Base, abstract.Sensor):
    """Represent an sensor in sqlalchemy."""

    __resource__ = 'sensors'
    __tablename__ = 'sensor'
    __table_args__ = sql.table_args()

    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True,
                           autoincrement=True)
    uuid = sqlalchemy.Column(sqlalchemy.String(36))
    project_id = sqlalchemy.Column(sqlalchemy.String(36))
    user_id = sqlalchemy.Column(sqlalchemy.String(36))
    name = sqlalchemy.Column(sqlalchemy.String(255))
    sensor_type = sqlalchemy.Column(sqlalchemy.String(255))
    value = sqlalchemy.Column(sqlalchemy.String(255))
    timestamp = sqlalchemy.Column(sqlalchemy.DateTime)
    description = sqlalchemy.Column(sqlalchemy.String(255))
    documentation = sqlalchemy.Column(sqlalchemy.String(255))
    target_resource = sqlalchemy.Column(sqlalchemy.String(255))


class SensorList(abstract.SensorList):
    """Represent a list of sensors in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return SensorList(sql.model_query(context, Sensor))



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import threading

from oslo_config import cfg
from oslo_db.sqlalchemy import session


_FACADE = None
_LOCK = threading.Lock()


def get_facade():
    global _LOCK, _FACADE
    if _FACADE is None:
        with _LOCK:
            if _FACADE is None:
                _FACADE = session.EngineFacade.from_config(cfg.CONF)
    return _FACADE


def get_engine():
    return get_facade().get_engine()


def get_session(autocommit=True, expire_on_commit=False):
    facade = get_facade()
    return facade.get_session(autocommit=autocommit,
                              expire_on_commit=expire_on_commit)


def get_backend():
    """The backend is this module itself."""
    return sys.modules[__name__]


def cleanup():
    global _FACADE

    if _FACADE:
        _FACADE.get_engine().dispose()
        _FACADE = None


def load():
    """Activate the sqlalchemy backend."""
    from solum import objects
    from solum.objects import app as abstract_app
    from solum.objects import assembly as abstract_assembly
    from solum.objects import component as abstract_component
    from solum.objects import execution as abstract_execution
    from solum.objects import extension as abstract_extension
    from solum.objects import image as abstract_image
    from solum.objects import infrastructure_stack as abstract_infra_stack
    from solum.objects import operation as abstract_operation
    from solum.objects import parameter as abstract_parameter
    from solum.objects import pipeline as abstract_pipeline
    from solum.objects import plan as abstract_plan
    from solum.objects import sensor as abstract_sensor
    from solum.objects import service as abstract_srvc
    from solum.objects.sqlalchemy import app
    from solum.objects.sqlalchemy import assembly
    from solum.objects.sqlalchemy import component
    from solum.objects.sqlalchemy import execution
    from solum.objects.sqlalchemy import extension
    from solum.objects.sqlalchemy import image
    from solum.objects.sqlalchemy import infrastructure_stack
    from solum.objects.sqlalchemy import operation
    from solum.objects.sqlalchemy import parameter
    from solum.objects.sqlalchemy import pipeline
    from solum.objects.sqlalchemy import plan
    from solum.objects.sqlalchemy import sensor
    from solum.objects.sqlalchemy import service
    from solum.objects.sqlalchemy import userlog
    from solum.objects.sqlalchemy import workflow
    from solum.objects import userlog as abstract_userlog
    from solum.objects import workflow as abstract_workflow

    objects.registry.add(abstract_app.App, app.App)
    objects.registry.add(abstract_app.AppList, app.AppList)
    objects.registry.add(abstract_assembly.Assembly, assembly.Assembly)
    objects.registry.add(abstract_assembly.AssemblyList, assembly.AssemblyList)
    objects.registry.add(abstract_infra_stack.InfrastructureStack,
                         infrastructure_stack.InfrastructureStack)
    objects.registry.add(abstract_infra_stack.InfrastructureStackList,
                         infrastructure_stack.InfrastructureStackList)
    objects.registry.add(abstract_component.Component, component.Component)
    objects.registry.add(abstract_component.ComponentList,
                         component.ComponentList)
    objects.registry.add(abstract_plan.Plan, plan.Plan)
    objects.registry.add(abstract_plan.PlanList, plan.PlanList)
    objects.registry.add(abstract_pipeline.Pipeline, pipeline.Pipeline)
    objects.registry.add(abstract_pipeline.PipelineList, pipeline.PipelineList)
    objects.registry.add(abstract_execution.Execution, execution.Execution)
    objects.registry.add(abstract_execution.ExecutionList,
                         execution.ExecutionList)
    objects.registry.add(abstract_srvc.Service, service.Service)
    objects.registry.add(abstract_srvc.ServiceList, service.ServiceList)
    objects.registry.add(abstract_operation.Operation, operation.Operation)
    objects.registry.add(abstract_operation.OperationList,
                         operation.OperationList)
    objects.registry.add(abstract_sensor.Sensor, sensor.Sensor)
    objects.registry.add(abstract_sensor.SensorList, sensor.SensorList)
    objects.registry.add(abstract_extension.Extension, extension.Extension)
    objects.registry.add(abstract_extension.ExtensionList,
                         extension.ExtensionList)
    objects.registry.add(abstract_image.Image, image.Image)
    objects.registry.add(abstract_image.ImageList, image.ImageList)
    objects.registry.add(abstract_userlog.Userlog, userlog.Userlog)
    objects.registry.add(abstract_userlog.UserlogList, userlog.UserlogList)
    objects.registry.add(abstract_parameter.Parameter, parameter.Parameter)
    objects.registry.add(abstract_parameter.ParameterList,
                         parameter.ParameterList)
    objects.registry.add(abstract_workflow.Workflow, workflow.Workflow)
    objects.registry.add(abstract_workflow.WorkflowList, workflow.WorkflowList)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.objects import registry
from solum.objects.sqlalchemy import infrastructure_stack
from solum.tests import base
from solum.tests import utils


class TestInfrastructureStack(base.BaseTestCase):
    def setUp(self):
        super(TestInfrastructureStack, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()

        self.data = [{'project_id': self.ctx.project_id,
                      'user_id': 'fred',
                      'uuid': 'ceda0408-c93d-4772-abb2-18f65189d440',
                      'name': 'o1',
                      'description': 'Scale up the resource',
                      'image_id': 'fake_image_id',
                      'heat_stack_id': 'fake_heat_stack_id'}]
        utils.create_models_from_data(infrastructure_stack.InfrastructureStack,
                                      self.data,
                                      self.ctx)

    def test_objects_registered(self):
        self.assertIsNotNone(registry.InfrastructureStack)
        self.assertIsNotNone(registry.InfrastructureStackList)

    def test_get_all(self):
        lst = infrastructure_stack.InfrastructureStackList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        pl = infrastructure_stack.InfrastructureStack().get_by_id(
            self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(pl, key))
